# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from coingeckopublicapiv3.api_helper import APIHelper


class SimpleTokenPrice(object):

    """Implementation of the 'SimpleTokenPrice' model.

    Attributes:
        usd (float): price in USD
        usd_market_cap (float): market cap in USD
        usd_24_h_vol (float): 24hr volume in USD
        usd_24_h_change (float): 24hr change in USD
        last_updated_at (float): last updated timestamp

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "usd": 'usd',
        "usd_market_cap": 'usd_market_cap',
        "usd_24_h_vol": 'usd_24h_vol',
        "usd_24_h_change": 'usd_24h_change',
        "last_updated_at": 'last_updated_at'
    }

    _optionals = [
        'usd',
        'usd_market_cap',
        'usd_24_h_vol',
        'usd_24_h_change',
        'last_updated_at',
    ]

    def __init__(self,
                 usd=APIHelper.SKIP,
                 usd_market_cap=APIHelper.SKIP,
                 usd_24_h_vol=APIHelper.SKIP,
                 usd_24_h_change=APIHelper.SKIP,
                 last_updated_at=APIHelper.SKIP):
        """Constructor for the SimpleTokenPrice class"""

        # Initialize members of the class
        if usd is not APIHelper.SKIP:
            self.usd = usd 
        if usd_market_cap is not APIHelper.SKIP:
            self.usd_market_cap = usd_market_cap 
        if usd_24_h_vol is not APIHelper.SKIP:
            self.usd_24_h_vol = usd_24_h_vol 
        if usd_24_h_change is not APIHelper.SKIP:
            self.usd_24_h_change = usd_24_h_change 
        if last_updated_at is not APIHelper.SKIP:
            self.last_updated_at = last_updated_at 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        usd = dictionary.get("usd") if dictionary.get("usd") else APIHelper.SKIP
        usd_market_cap = dictionary.get("usd_market_cap") if dictionary.get("usd_market_cap") else APIHelper.SKIP
        usd_24_h_vol = dictionary.get("usd_24h_vol") if dictionary.get("usd_24h_vol") else APIHelper.SKIP
        usd_24_h_change = dictionary.get("usd_24h_change") if dictionary.get("usd_24h_change") else APIHelper.SKIP
        last_updated_at = dictionary.get("last_updated_at") if dictionary.get("last_updated_at") else APIHelper.SKIP
        # Return an object of this model
        return cls(usd,
                   usd_market_cap,
                   usd_24_h_vol,
                   usd_24_h_change,
                   last_updated_at)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'usd={(self.usd if hasattr(self, "usd") else None)!r}, '
                f'usd_market_cap={(self.usd_market_cap if hasattr(self, "usd_market_cap") else None)!r}, '
                f'usd_24_h_vol={(self.usd_24_h_vol if hasattr(self, "usd_24_h_vol") else None)!r}, '
                f'usd_24_h_change={(self.usd_24_h_change if hasattr(self, "usd_24_h_change") else None)!r}, '
                f'last_updated_at={(self.last_updated_at if hasattr(self, "last_updated_at") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'usd={(self.usd if hasattr(self, "usd") else None)!s}, '
                f'usd_market_cap={(self.usd_market_cap if hasattr(self, "usd_market_cap") else None)!s}, '
                f'usd_24_h_vol={(self.usd_24_h_vol if hasattr(self, "usd_24_h_vol") else None)!s}, '
                f'usd_24_h_change={(self.usd_24_h_change if hasattr(self, "usd_24_h_change") else None)!s}, '
                f'last_updated_at={(self.last_updated_at if hasattr(self, "last_updated_at") else None)!s})')



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from coingeckopublicapiv3.api_helper import APIHelper


class ReposUrl(object):

    """Implementation of the 'ReposUrl' model.

    coin repository url

    Attributes:
        github (List[str]): coin github repository url
        bitbucket (List[str]): coin bitbucket repository url

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "github": 'github',
        "bitbucket": 'bitbucket'
    }

    _optionals = [
        'github',
        'bitbucket',
    ]

    def __init__(self,
                 github=APIHelper.SKIP,
                 bitbucket=APIHelper.SKIP):
        """Constructor for the ReposUrl class"""

        # Initialize members of the class
        if github is not APIHelper.SKIP:
            self.github = github 
        if bitbucket is not APIHelper.SKIP:
            self.bitbucket = bitbucket 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        github = dictionary.get("github") if dictionary.get("github") else APIHelper.SKIP
        bitbucket = dictionary.get("bitbucket") if dictionary.get("bitbucket") else APIHelper.SKIP
        # Return an object of this model
        return cls(github,
                   bitbucket)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'github={(self.github if hasattr(self, "github") else None)!r}, '
                f'bitbucket={(self.bitbucket if hasattr(self, "bitbucket") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'github={(self.github if hasattr(self, "github") else None)!s}, '
                f'bitbucket={(self.bitbucket if hasattr(self, "bitbucket") else None)!s})')



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from coingeckopublicapiv3.api_helper import APIHelper
from coingeckopublicapiv3.models.community_data_1 import CommunityData1
from coingeckopublicapiv3.models.developer_data_1 import DeveloperData1
from coingeckopublicapiv3.models.image_1 import Image1
from coingeckopublicapiv3.models.market_data_1 import MarketData1
from coingeckopublicapiv3.models.public_interest_stats import PublicInterestStats


class CoinsHistoricalData(object):

    """Implementation of the 'CoinsHistoricalData' model.

    Attributes:
        id (str): coin ID
        symbol (str): coin symbol
        name (str): coin name
        localization (Dict[str, str]): coin localization
        image (Image1): coin image url
        market_data (MarketData1): coin market data
        community_data (CommunityData1): coin community data
        developer_data (DeveloperData1): coin developer data
        public_interest_stats (PublicInterestStats): coin public interest stats

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id": 'id',
        "symbol": 'symbol',
        "name": 'name',
        "localization": 'localization',
        "image": 'image',
        "market_data": 'market_data',
        "community_data": 'community_data',
        "developer_data": 'developer_data',
        "public_interest_stats": 'public_interest_stats'
    }

    _optionals = [
        'id',
        'symbol',
        'name',
        'localization',
        'image',
        'market_data',
        'community_data',
        'developer_data',
        'public_interest_stats',
    ]

    def __init__(self,
                 id=APIHelper.SKIP,
                 symbol=APIHelper.SKIP,
                 name=APIHelper.SKIP,
                 localization=APIHelper.SKIP,
                 image=APIHelper.SKIP,
                 market_data=APIHelper.SKIP,
                 community_data=APIHelper.SKIP,
                 developer_data=APIHelper.SKIP,
                 public_interest_stats=APIHelper.SKIP):
        """Constructor for the CoinsHistoricalData class"""

        # Initialize members of the class
        if id is not APIHelper.SKIP:
            self.id = id 
        if symbol is not APIHelper.SKIP:
            self.symbol = symbol 
        if name is not APIHelper.SKIP:
            self.name = name 
        if localization is not APIHelper.SKIP:
            self.localization = localization 
        if image is not APIHelper.SKIP:
            self.image = image 
        if market_data is not APIHelper.SKIP:
            self.market_data = market_data 
        if community_data is not APIHelper.SKIP:
            self.community_data = community_data 
        if developer_data is not APIHelper.SKIP:
            self.developer_data = developer_data 
        if public_interest_stats is not APIHelper.SKIP:
            self.public_interest_stats = public_interest_stats 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id") if dictionary.get("id") else APIHelper.SKIP
        symbol = dictionary.get("symbol") if dictionary.get("symbol") else APIHelper.SKIP
        name = dictionary.get("name") if dictionary.get("name") else APIHelper.SKIP
        localization = dictionary.get("localization") if dictionary.get("localization") else APIHelper.SKIP
        image = Image1.from_dictionary(dictionary.get('image')) if 'image' in dictionary.keys() else APIHelper.SKIP
        market_data = MarketData1.from_dictionary(dictionary.get('market_data')) if 'market_data' in dictionary.keys() else APIHelper.SKIP
        community_data = CommunityData1.from_dictionary(dictionary.get('community_data')) if 'community_data' in dictionary.keys() else APIHelper.SKIP
        developer_data = DeveloperData1.from_dictionary(dictionary.get('developer_data')) if 'developer_data' in dictionary.keys() else APIHelper.SKIP
        public_interest_stats = PublicInterestStats.from_dictionary(dictionary.get('public_interest_stats')) if 'public_interest_stats' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(id,
                   symbol,
                   name,
                   localization,
                   image,
                   market_data,
                   community_data,
                   developer_data,
                   public_interest_stats)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'id={(self.id if hasattr(self, "id") else None)!r}, '
                f'symbol={(self.symbol if hasattr(self, "symbol") else None)!r}, '
                f'name={(self.name if hasattr(self, "name") else None)!r}, '
                f'localization={(self.localization if hasattr(self, "localization") else None)!r}, '
                f'image={(self.image if hasattr(self, "image") else None)!r}, '
                f'market_data={(self.market_data if hasattr(self, "market_data") else None)!r}, '
                f'community_data={(self.community_data if hasattr(self, "community_data") else None)!r}, '
                f'developer_data={(self.developer_data if hasattr(self, "developer_data") else None)!r}, '
                f'public_interest_stats={(self.public_interest_stats if hasattr(self, "public_interest_stats") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'id={(self.id if hasattr(self, "id") else None)!s}, '
                f'symbol={(self.symbol if hasattr(self, "symbol") else None)!s}, '
                f'name={(self.name if hasattr(self, "name") else None)!s}, '
                f'localization={(self.localization if hasattr(self, "localization") else None)!s}, '
                f'image={(self.image if hasattr(self, "image") else None)!s}, '
                f'market_data={(self.market_data if hasattr(self, "market_data") else None)!s}, '
                f'community_data={(self.community_data if hasattr(self, "community_data") else None)!s}, '
                f'developer_data={(self.developer_data if hasattr(self, "developer_data") else None)!s}, '
                f'public_interest_stats={(self.public_interest_stats if hasattr(self, "public_interest_stats") else None)!s})')



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from coingeckopublicapiv3.api_helper import APIHelper
from coingeckopublicapiv3.models.converted_last import ConvertedLast
from coingeckopublicapiv3.models.converted_volume import ConvertedVolume
from coingeckopublicapiv3.models.market_1 import Market1


class Ticker1(object):

    """Implementation of the 'Ticker1' model.

    Attributes:
        base (str): coin ticker base currency
        target (str): coin ticker target currency
        market (Market1): coin ticker exchange
        last (float): coin ticker last price
        volume (float): coin ticker volume
        cost_to_move_up_usd (float): coin ticker cost to move up in usd
        cost_to_move_down_usd (float): coin ticker cost to move down in usd
        converted_last (ConvertedLast): coin ticker converted last price
        converted_volume (ConvertedVolume): coin ticker converted volume
        trust_score (str): coin ticker trust score
        bid_ask_spread_percentage (float): coin ticker bid ask spread
            percentage
        timestamp (str): coin ticker timestamp
        last_traded_at (str): coin ticker last traded timestamp
        last_fetch_at (str): coin ticker last fetch timestamp
        is_anomaly (bool): coin ticker anomaly
        is_stale (bool): coin ticker stale
        trade_url (str): coin ticker trade url
        token_info_url (str): coin ticker token info url
        coin_id (str): coin ticker base currency coin ID
        target_coin_id (str): coin ticker target currency coin ID

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "base": 'base',
        "target": 'target',
        "market": 'market',
        "last": 'last',
        "volume": 'volume',
        "cost_to_move_up_usd": 'cost_to_move_up_usd',
        "cost_to_move_down_usd": 'cost_to_move_down_usd',
        "converted_last": 'converted_last',
        "converted_volume": 'converted_volume',
        "trust_score": 'trust_score',
        "bid_ask_spread_percentage": 'bid_ask_spread_percentage',
        "timestamp": 'timestamp',
        "last_traded_at": 'last_traded_at',
        "last_fetch_at": 'last_fetch_at',
        "is_anomaly": 'is_anomaly',
        "is_stale": 'is_stale',
        "trade_url": 'trade_url',
        "token_info_url": 'token_info_url',
        "coin_id": 'coin_id',
        "target_coin_id": 'target_coin_id'
    }

    _optionals = [
        'base',
        'target',
        'market',
        'last',
        'volume',
        'cost_to_move_up_usd',
        'cost_to_move_down_usd',
        'converted_last',
        'converted_volume',
        'trust_score',
        'bid_ask_spread_percentage',
        'timestamp',
        'last_traded_at',
        'last_fetch_at',
        'is_anomaly',
        'is_stale',
        'trade_url',
        'token_info_url',
        'coin_id',
        'target_coin_id',
    ]

    _nullables = [
        'token_info_url',
    ]

    def __init__(self,
                 base=APIHelper.SKIP,
                 target=APIHelper.SKIP,
                 market=APIHelper.SKIP,
                 last=APIHelper.SKIP,
                 volume=APIHelper.SKIP,
                 cost_to_move_up_usd=APIHelper.SKIP,
                 cost_to_move_down_usd=APIHelper.SKIP,
                 converted_last=APIHelper.SKIP,
                 converted_volume=APIHelper.SKIP,
                 trust_score=APIHelper.SKIP,
                 bid_ask_spread_percentage=APIHelper.SKIP,
                 timestamp=APIHelper.SKIP,
                 last_traded_at=APIHelper.SKIP,
                 last_fetch_at=APIHelper.SKIP,
                 is_anomaly=APIHelper.SKIP,
                 is_stale=APIHelper.SKIP,
                 trade_url=APIHelper.SKIP,
                 token_info_url=APIHelper.SKIP,
                 coin_id=APIHelper.SKIP,
                 target_coin_id=APIHelper.SKIP):
        """Constructor for the Ticker1 class"""

        # Initialize members of the class
        if base is not APIHelper.SKIP:
            self.base = base 
        if target is not APIHelper.SKIP:
            self.target = target 
        if market is not APIHelper.SKIP:
            self.market = market 
        if last is not APIHelper.SKIP:
            self.last = last 
        if volume is not APIHelper.SKIP:
            self.volume = volume 
        if cost_to_move_up_usd is not APIHelper.SKIP:
            self.cost_to_move_up_usd = cost_to_move_up_usd 
        if cost_to_move_down_usd is not APIHelper.SKIP:
            self.cost_to_move_down_usd = cost_to_move_down_usd 
        if converted_last is not APIHelper.SKIP:
            self.converted_last = converted_last 
        if converted_volume is not APIHelper.SKIP:
            self.converted_volume = converted_volume 
        if trust_score is not APIHelper.SKIP:
            self.trust_score = trust_score 
        if bid_ask_spread_percentage is not APIHelper.SKIP:
            self.bid_ask_spread_percentage = bid_ask_spread_percentage 
        if timestamp is not APIHelper.SKIP:
            self.timestamp = timestamp 
        if last_traded_at is not APIHelper.SKIP:
            self.last_traded_at = last_traded_at 
        if last_fetch_at is not APIHelper.SKIP:
            self.last_fetch_at = last_fetch_at 
        if is_anomaly is not APIHelper.SKIP:
            self.is_anomaly = is_anomaly 
        if is_stale is not APIHelper.SKIP:
            self.is_stale = is_stale 
        if trade_url is not APIHelper.SKIP:
            self.trade_url = trade_url 
        if token_info_url is not APIHelper.SKIP:
            self.token_info_url = token_info_url 
        if coin_id is not APIHelper.SKIP:
            self.coin_id = coin_id 
        if target_coin_id is not APIHelper.SKIP:
            self.target_coin_id = target_coin_id 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        base = dictionary.get("base") if dictionary.get("base") else APIHelper.SKIP
        target = dictionary.get("target") if dictionary.get("target") else APIHelper.SKIP
        market = Market1.from_dictionary(dictionary.get('market')) if 'market' in dictionary.keys() else APIHelper.SKIP
        last = dictionary.get("last") if dictionary.get("last") else APIHelper.SKIP
        volume = dictionary.get("volume") if dictionary.get("volume") else APIHelper.SKIP
        cost_to_move_up_usd = dictionary.get("cost_to_move_up_usd") if dictionary.get("cost_to_move_up_usd") else APIHelper.SKIP
        cost_to_move_down_usd = dictionary.get("cost_to_move_down_usd") if dictionary.get("cost_to_move_down_usd") else APIHelper.SKIP
        converted_last = ConvertedLast.from_dictionary(dictionary.get('converted_last')) if 'converted_last' in dictionary.keys() else APIHelper.SKIP
        converted_volume = ConvertedVolume.from_dictionary(dictionary.get('converted_volume')) if 'converted_volume' in dictionary.keys() else APIHelper.SKIP
        trust_score = dictionary.get("trust_score") if dictionary.get("trust_score") else APIHelper.SKIP
        bid_ask_spread_percentage = dictionary.get("bid_ask_spread_percentage") if dictionary.get("bid_ask_spread_percentage") else APIHelper.SKIP
        timestamp = dictionary.get("timestamp") if dictionary.get("timestamp") else APIHelper.SKIP
        last_traded_at = dictionary.get("last_traded_at") if dictionary.get("last_traded_at") else APIHelper.SKIP
        last_fetch_at = dictionary.get("last_fetch_at") if dictionary.get("last_fetch_at") else APIHelper.SKIP
        is_anomaly = dictionary.get("is_anomaly") if "is_anomaly" in dictionary.keys() else APIHelper.SKIP
        is_stale = dictionary.get("is_stale") if "is_stale" in dictionary.keys() else APIHelper.SKIP
        trade_url = dictionary.get("trade_url") if dictionary.get("trade_url") else APIHelper.SKIP
        token_info_url = dictionary.get("token_info_url") if "token_info_url" in dictionary.keys() else APIHelper.SKIP
        coin_id = dictionary.get("coin_id") if dictionary.get("coin_id") else APIHelper.SKIP
        target_coin_id = dictionary.get("target_coin_id") if dictionary.get("target_coin_id") else APIHelper.SKIP
        # Return an object of this model
        return cls(base,
                   target,
                   market,
                   last,
                   volume,
                   cost_to_move_up_usd,
                   cost_to_move_down_usd,
                   converted_last,
                   converted_volume,
                   trust_score,
                   bid_ask_spread_percentage,
                   timestamp,
                   last_traded_at,
                   last_fetch_at,
                   is_anomaly,
                   is_stale,
                   trade_url,
                   token_info_url,
                   coin_id,
                   target_coin_id)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'base={(self.base if hasattr(self, "base") else None)!r}, '
                f'target={(self.target if hasattr(self, "target") else None)!r}, '
                f'market={(self.market if hasattr(self, "market") else None)!r}, '
                f'last={(self.last if hasattr(self, "last") else None)!r}, '
                f'volume={(self.volume if hasattr(self, "volume") else None)!r}, '
                f'cost_to_move_up_usd={(self.cost_to_move_up_usd if hasattr(self, "cost_to_move_up_usd") else None)!r}, '
                f'cost_to_move_down_usd={(self.cost_to_move_down_usd if hasattr(self, "cost_to_move_down_usd") else None)!r}, '
                f'converted_last={(self.converted_last if hasattr(self, "converted_last") else None)!r}, '
                f'converted_volume={(self.converted_volume if hasattr(self, "converted_volume") else None)!r}, '
                f'trust_score={(self.trust_score if hasattr(self, "trust_score") else None)!r}, '
                f'bid_ask_spread_percentage={(self.bid_ask_spread_percentage if hasattr(self, "bid_ask_spread_percentage") else None)!r}, '
                f'timestamp={(self.timestamp if hasattr(self, "timestamp") else None)!r}, '
                f'last_traded_at={(self.last_traded_at if hasattr(self, "last_traded_at") else None)!r}, '
                f'last_fetch_at={(self.last_fetch_at if hasattr(self, "last_fetch_at") else None)!r}, '
                f'is_anomaly={(self.is_anomaly if hasattr(self, "is_anomaly") else None)!r}, '
                f'is_stale={(self.is_stale if hasattr(self, "is_stale") else None)!r}, '
                f'trade_url={(self.trade_url if hasattr(self, "trade_url") else None)!r}, '
                f'token_info_url={(self.token_info_url if hasattr(self, "token_info_url") else None)!r}, '
                f'coin_id={(self.coin_id if hasattr(self, "coin_id") else None)!r}, '
                f'target_coin_id={(self.target_coin_id if hasattr(self, "target_coin_id") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'base={(self.base if hasattr(self, "base") else None)!s}, '
                f'target={(self.target if hasattr(self, "target") else None)!s}, '
                f'market={(self.market if hasattr(self, "market") else None)!s}, '
                f'last={(self.last if hasattr(self, "last") else None)!s}, '
                f'volume={(self.volume if hasattr(self, "volume") else None)!s}, '
                f'cost_to_move_up_usd={(self.cost_to_move_up_usd if hasattr(self, "cost_to_move_up_usd") else None)!s}, '
                f'cost_to_move_down_usd={(self.cost_to_move_down_usd if hasattr(self, "cost_to_move_down_usd") else None)!s}, '
                f'converted_last={(self.converted_last if hasattr(self, "converted_last") else None)!s}, '
                f'converted_volume={(self.converted_volume if hasattr(self, "converted_volume") else None)!s}, '
                f'trust_score={(self.trust_score if hasattr(self, "trust_score") else None)!s}, '
                f'bid_ask_spread_percentage={(self.bid_ask_spread_percentage if hasattr(self, "bid_ask_spread_percentage") else None)!s}, '
                f'timestamp={(self.timestamp if hasattr(self, "timestamp") else None)!s}, '
                f'last_traded_at={(self.last_traded_at if hasattr(self, "last_traded_at") else None)!s}, '
                f'last_fetch_at={(self.last_fetch_at if hasattr(self, "last_fetch_at") else None)!s}, '
                f'is_anomaly={(self.is_anomaly if hasattr(self, "is_anomaly") else None)!s}, '
                f'is_stale={(self.is_stale if hasattr(self, "is_stale") else None)!s}, '
                f'trade_url={(self.trade_url if hasattr(self, "trade_url") else None)!s}, '
                f'token_info_url={(self.token_info_url if hasattr(self, "token_info_url") else None)!s}, '
                f'coin_id={(self.coin_id if hasattr(self, "coin_id") else None)!s}, '
                f'target_coin_id={(self.target_coin_id if hasattr(self, "target_coin_id") else None)!s})')



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from coingeckopublicapiv3.api_helper import APIHelper
from coingeckopublicapiv3.configuration import Server
from coingeckopublicapiv3.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from coingeckopublicapiv3.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from apimatic_core.authentication.multiple.or_auth_group import Or
from coingeckopublicapiv3.models.exchanges import Exchanges
from coingeckopublicapiv3.models.exchanges_list import ExchangesList
from coingeckopublicapiv3.models.exchange_data import ExchangeData
from coingeckopublicapiv3.models.coins_tickers import CoinsTickers


class ExchangesController(BaseController):

    """A Controller to access Endpoints in the coingeckopublicapiv3 API."""
    def __init__(self, config):
        super(ExchangesController, self).__init__(config)

    def exchanges(self,
                  per_page=None,
                  page=None):
        """Does a GET request to /exchanges.

        This endpoint allows you to **query all the supported exchanges with
        exchanges’ data (ID, name, country, ...) that have active trading
        volumes on CoinGecko**

        Args:
            per_page (int, optional): total results per page, default: 100
                <br> Valid values: 1...250
            page (int, optional): page through results, default: 1

        Returns:
            Exchanges: Response from the API. List all exchanges

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/exchanges')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('per_page')
                         .value(per_page))
            .query_param(Parameter()
                         .key('page')
                         .value(page))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Or(Single('apiKeyAuth'), Single('apiKeyQueryParam')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Exchanges.from_dictionary)
        ).execute()

    def exchanges_list(self,
                       status=None):
        """Does a GET request to /exchanges/list.

        This endpoint allows you to **query all the exchanges with ID and
        name**

        Args:
            status (StatusEnum, optional): filter by status of exchanges,
                default: active

        Returns:
            ExchangesList: Response from the API. List all exchanges with ID
                and name

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/exchanges/list')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('status')
                         .value(status))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Or(Single('apiKeyAuth'), Single('apiKeyQueryParam')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ExchangesList.from_dictionary)
        ).execute()

    def exchanges_id(self,
                     id,
                     dex_pair_format=None):
        """Does a GET request to /exchanges/{id}.

        This endpoint allows you to **query exchange’s data (name, year
        established, country, ...), exchange volume in BTC and top 100 tickers
        based on exchange’s ID**

        Args:
            id (str): exchange ID <br> *refers to
                [`/exchanges/list`](/reference/exchanges-list).
            dex_pair_format (DexPairFormatEnum, optional): set to `symbol` to
                display DEX pair base and target as symbols, default:
                `contract_address`

        Returns:
            ExchangeData: Response from the API. Get exchange volume in BTC
                and top 100 tickers only

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/exchanges/{id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('dex_pair_format')
                         .value(dex_pair_format))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Or(Single('apiKeyAuth'), Single('apiKeyQueryParam')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ExchangeData.from_dictionary)
        ).execute()

    def exchanges_id_tickers(self,
                             id,
                             coin_ids=None,
                             include_exchange_logo=None,
                             page=None,
                             depth=None,
                             order=None,
                             dex_pair_format=None):
        """Does a GET request to /exchanges/{id}/tickers.

        This endpoint allows you to **query exchange's tickers based on
        exchange’s ID**

        Args:
            id (str): exchange ID <br> *refers to
                [`/exchanges/list`](/reference/exchanges-list).
            coin_ids (str, optional): filter tickers by coin IDs,
                comma-separated if querying more than 1 coin <br> *refers to
                [`/coins/list`](/reference/coins-list).
            include_exchange_logo (bool, optional): include exchange logo,
                default: false
            page (int, optional): page through results
            depth (bool, optional): include 2% orderbook depth (Example:
                cost_to_move_up_usd & cost_to_move_down_usd),default: false
            order (Order3Enum, optional): use this to sort the order of
                responses, default: trust_score_desc
            dex_pair_format (DexPairFormatEnum, optional): set to `symbol` to
                display DEX pair base and target as symbols, default:
                `contract_address`

        Returns:
            CoinsTickers: Response from the API. Get exchange tickers

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/exchanges/{id}/tickers')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('coin_ids')
                         .value(coin_ids))
            .query_param(Parameter()
                         .key('include_exchange_logo')
                         .value(include_exchange_logo))
            .query_param(Parameter()
                         .key('page')
                         .value(page))
            .query_param(Parameter()
                         .key('depth')
                         .value(depth))
            .query_param(Parameter()
                         .key('order')
                         .value(order))
            .query_param(Parameter()
                         .key('dex_pair_format')
                         .value(dex_pair_format))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Or(Single('apiKeyAuth'), Single('apiKeyQueryParam')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CoinsTickers.from_dictionary)
        ).execute()

    def exchanges_id_volume_chart(self,
                                  id,
                                  days):
        """Does a GET request to /exchanges/{id}/volume_chart.

        This endpoint allows you to **query the historical volume chart data
        with time in UNIX and trading volume data in BTC based on exchange’s
        ID**

        Args:
            id (str): exchange ID or derivatives exchange ID <br> *refers to
                [`/exchanges/list`](/reference/exchanges-list) or
                [`/derivatives/exchanges/list`](/reference/derivatives-exchange
                s-list).
            days (DaysEnum): data up to number of days ago

        Returns:
            List[float]: Response from the API. Get exchange volume chart data

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/exchanges/{id}/volume_chart')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('days')
                         .value(days))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Or(Single('apiKeyAuth'), Single('apiKeyQueryParam')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
        ).execute()



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from coingeckopublicapiv3.api_helper import APIHelper


class Token(object):

    """Implementation of the 'Token' model.

    Attributes:
        chain_id (float): chainlist's chain ID
        address (str): token contract address
        name (str): token name
        symbol (str): token symbol
        decimals (float): token decimals
        logo_uri (str): token image url

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "chain_id": 'chainId',
        "address": 'address',
        "name": 'name',
        "symbol": 'symbol',
        "decimals": 'decimals',
        "logo_uri": 'logoURI'
    }

    _optionals = [
        'chain_id',
        'address',
        'name',
        'symbol',
        'decimals',
        'logo_uri',
    ]

    def __init__(self,
                 chain_id=APIHelper.SKIP,
                 address=APIHelper.SKIP,
                 name=APIHelper.SKIP,
                 symbol=APIHelper.SKIP,
                 decimals=APIHelper.SKIP,
                 logo_uri=APIHelper.SKIP):
        """Constructor for the Token class"""

        # Initialize members of the class
        if chain_id is not APIHelper.SKIP:
            self.chain_id = chain_id 
        if address is not APIHelper.SKIP:
            self.address = address 
        if name is not APIHelper.SKIP:
            self.name = name 
        if symbol is not APIHelper.SKIP:
            self.symbol = symbol 
        if decimals is not APIHelper.SKIP:
            self.decimals = decimals 
        if logo_uri is not APIHelper.SKIP:
            self.logo_uri = logo_uri 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        chain_id = dictionary.get("chainId") if dictionary.get("chainId") else APIHelper.SKIP
        address = dictionary.get("address") if dictionary.get("address") else APIHelper.SKIP
        name = dictionary.get("name") if dictionary.get("name") else APIHelper.SKIP
        symbol = dictionary.get("symbol") if dictionary.get("symbol") else APIHelper.SKIP
        decimals = dictionary.get("decimals") if dictionary.get("decimals") else APIHelper.SKIP
        logo_uri = dictionary.get("logoURI") if dictionary.get("logoURI") else APIHelper.SKIP
        # Return an object of this model
        return cls(chain_id,
                   address,
                   name,
                   symbol,
                   decimals,
                   logo_uri)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'chain_id={(self.chain_id if hasattr(self, "chain_id") else None)!r}, '
                f'address={(self.address if hasattr(self, "address") else None)!r}, '
                f'name={(self.name if hasattr(self, "name") else None)!r}, '
                f'symbol={(self.symbol if hasattr(self, "symbol") else None)!r}, '
                f'decimals={(self.decimals if hasattr(self, "decimals") else None)!r}, '
                f'logo_uri={(self.logo_uri if hasattr(self, "logo_uri") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'chain_id={(self.chain_id if hasattr(self, "chain_id") else None)!s}, '
                f'address={(self.address if hasattr(self, "address") else None)!s}, '
                f'name={(self.name if hasattr(self, "name") else None)!s}, '
                f'symbol={(self.symbol if hasattr(self, "symbol") else None)!s}, '
                f'decimals={(self.decimals if hasattr(self, "decimals") else None)!s}, '
                f'logo_uri={(self.logo_uri if hasattr(self, "logo_uri") else None)!s})')



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from coingeckopublicapiv3.api_helper import APIHelper
from coingeckopublicapiv3.models.repos_url import ReposUrl


class Links(object):

    """Implementation of the 'Links' model.

    links

    Attributes:
        homepage (List[str]): coin website url
        whitepaper (List[str]): coin whitepaper url
        blockchain_site (List[str]): coin block explorer url
        official_forum_url (List[str]): coin official forum url
        chat_url (List[str]): coin chat url
        announcement_url (List[str]): coin announcement url
        snapshot_url (str): coin snapshot url
        twitter_screen_name (str): coin twitter handle
        facebook_username (str): coin facebook username
        bitcointalk_thread_identifier (str): coin bitcointalk thread identifier
        telegram_channel_identifier (str): coin telegram channel identifier
        subreddit_url (str): coin subreddit url
        repos_url (ReposUrl): coin repository url

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "homepage": 'homepage',
        "whitepaper": 'whitepaper',
        "blockchain_site": 'blockchain_site',
        "official_forum_url": 'official_forum_url',
        "chat_url": 'chat_url',
        "announcement_url": 'announcement_url',
        "snapshot_url": 'snapshot_url',
        "twitter_screen_name": 'twitter_screen_name',
        "facebook_username": 'facebook_username',
        "bitcointalk_thread_identifier": 'bitcointalk_thread_identifier',
        "telegram_channel_identifier": 'telegram_channel_identifier',
        "subreddit_url": 'subreddit_url',
        "repos_url": 'repos_url'
    }

    _optionals = [
        'homepage',
        'whitepaper',
        'blockchain_site',
        'official_forum_url',
        'chat_url',
        'announcement_url',
        'snapshot_url',
        'twitter_screen_name',
        'facebook_username',
        'bitcointalk_thread_identifier',
        'telegram_channel_identifier',
        'subreddit_url',
        'repos_url',
    ]

    def __init__(self,
                 homepage=APIHelper.SKIP,
                 whitepaper=APIHelper.SKIP,
                 blockchain_site=APIHelper.SKIP,
                 official_forum_url=APIHelper.SKIP,
                 chat_url=APIHelper.SKIP,
                 announcement_url=APIHelper.SKIP,
                 snapshot_url=APIHelper.SKIP,
                 twitter_screen_name=APIHelper.SKIP,
                 facebook_username=APIHelper.SKIP,
                 bitcointalk_thread_identifier=APIHelper.SKIP,
                 telegram_channel_identifier=APIHelper.SKIP,
                 subreddit_url=APIHelper.SKIP,
                 repos_url=APIHelper.SKIP):
        """Constructor for the Links class"""

        # Initialize members of the class
        if homepage is not APIHelper.SKIP:
            self.homepage = homepage 
        if whitepaper is not APIHelper.SKIP:
            self.whitepaper = whitepaper 
        if blockchain_site is not APIHelper.SKIP:
            self.blockchain_site = blockchain_site 
        if official_forum_url is not APIHelper.SKIP:
            self.official_forum_url = official_forum_url 
        if chat_url is not APIHelper.SKIP:
            self.chat_url = chat_url 
        if announcement_url is not APIHelper.SKIP:
            self.announcement_url = announcement_url 
        if snapshot_url is not APIHelper.SKIP:
            self.snapshot_url = snapshot_url 
        if twitter_screen_name is not APIHelper.SKIP:
            self.twitter_screen_name = twitter_screen_name 
        if facebook_username is not APIHelper.SKIP:
            self.facebook_username = facebook_username 
        if bitcointalk_thread_identifier is not APIHelper.SKIP:
            self.bitcointalk_thread_identifier = bitcointalk_thread_identifier 
        if telegram_channel_identifier is not APIHelper.SKIP:
            self.telegram_channel_identifier = telegram_channel_identifier 
        if subreddit_url is not APIHelper.SKIP:
            self.subreddit_url = subreddit_url 
        if repos_url is not APIHelper.SKIP:
            self.repos_url = repos_url 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        homepage = dictionary.get("homepage") if dictionary.get("homepage") else APIHelper.SKIP
        whitepaper = dictionary.get("whitepaper") if dictionary.get("whitepaper") else APIHelper.SKIP
        blockchain_site = dictionary.get("blockchain_site") if dictionary.get("blockchain_site") else APIHelper.SKIP
        official_forum_url = dictionary.get("official_forum_url") if dictionary.get("official_forum_url") else APIHelper.SKIP
        chat_url = dictionary.get("chat_url") if dictionary.get("chat_url") else APIHelper.SKIP
        announcement_url = dictionary.get("announcement_url") if dictionary.get("announcement_url") else APIHelper.SKIP
        snapshot_url = dictionary.get("snapshot_url") if dictionary.get("snapshot_url") else APIHelper.SKIP
        twitter_screen_name = dictionary.get("twitter_screen_name") if dictionary.get("twitter_screen_name") else APIHelper.SKIP
        facebook_username = dictionary.get("facebook_username") if dictionary.get("facebook_username") else APIHelper.SKIP
        bitcointalk_thread_identifier = dictionary.get("bitcointalk_thread_identifier") if dictionary.get("bitcointalk_thread_identifier") else APIHelper.SKIP
        telegram_channel_identifier = dictionary.get("telegram_channel_identifier") if dictionary.get("telegram_channel_identifier") else APIHelper.SKIP
        subreddit_url = dictionary.get("subreddit_url") if dictionary.get("subreddit_url") else APIHelper.SKIP
        repos_url = ReposUrl.from_dictionary(dictionary.get('repos_url')) if 'repos_url' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(homepage,
                   whitepaper,
                   blockchain_site,
                   official_forum_url,
                   chat_url,
                   announcement_url,
                   snapshot_url,
                   twitter_screen_name,
                   facebook_username,
                   bitcointalk_thread_identifier,
                   telegram_channel_identifier,
                   subreddit_url,
                   repos_url)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'homepage={(self.homepage if hasattr(self, "homepage") else None)!r}, '
                f'whitepaper={(self.whitepaper if hasattr(self, "whitepaper") else None)!r}, '
                f'blockchain_site={(self.blockchain_site if hasattr(self, "blockchain_site") else None)!r}, '
                f'official_forum_url={(self.official_forum_url if hasattr(self, "official_forum_url") else None)!r}, '
                f'chat_url={(self.chat_url if hasattr(self, "chat_url") else None)!r}, '
                f'announcement_url={(self.announcement_url if hasattr(self, "announcement_url") else None)!r}, '
                f'snapshot_url={(self.snapshot_url if hasattr(self, "snapshot_url") else None)!r}, '
                f'twitter_screen_name={(self.twitter_screen_name if hasattr(self, "twitter_screen_name") else None)!r}, '
                f'facebook_username={(self.facebook_username if hasattr(self, "facebook_username") else None)!r}, '
                f'bitcointalk_thread_identifier={(self.bitcointalk_thread_identifier if hasattr(self, "bitcointalk_thread_identifier") else None)!r}, '
                f'telegram_channel_identifier={(self.telegram_channel_identifier if hasattr(self, "telegram_channel_identifier") else None)!r}, '
                f'subreddit_url={(self.subreddit_url if hasattr(self, "subreddit_url") else None)!r}, '
                f'repos_url={(self.repos_url if hasattr(self, "repos_url") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'homepage={(self.homepage if hasattr(self, "homepage") else None)!s}, '
                f'whitepaper={(self.whitepaper if hasattr(self, "whitepaper") else None)!s}, '
                f'blockchain_site={(self.blockchain_site if hasattr(self, "blockchain_site") else None)!s}, '
                f'official_forum_url={(self.official_forum_url if hasattr(self, "official_forum_url") else None)!s}, '
                f'chat_url={(self.chat_url if hasattr(self, "chat_url") else None)!s}, '
                f'announcement_url={(self.announcement_url if hasattr(self, "announcement_url") else None)!s}, '
                f'snapshot_url={(self.snapshot_url if hasattr(self, "snapshot_url") else None)!s}, '
                f'twitter_screen_name={(self.twitter_screen_name if hasattr(self, "twitter_screen_name") else None)!s}, '
                f'facebook_username={(self.facebook_username if hasattr(self, "facebook_username") else None)!s}, '
                f'bitcointalk_thread_identifier={(self.bitcointalk_thread_identifier if hasattr(self, "bitcointalk_thread_identifier") else None)!s}, '
                f'telegram_channel_identifier={(self.telegram_channel_identifier if hasattr(self, "telegram_channel_identifier") else None)!s}, '
                f'subreddit_url={(self.subreddit_url if hasattr(self, "subreddit_url") else None)!s}, '
                f'repos_url={(self.repos_url if hasattr(self, "repos_url") else None)!s})')



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from coingeckopublicapiv3.api_helper import APIHelper


class Coin(object):

    """Implementation of the 'Coin' model.

    Attributes:
        id (str): coin ID
        name (str): coin name
        api_symbol (str): coin api symbol
        symbol (str): coin symbol
        market_cap_rank (float): coin market cap rank
        thumb (str): coin thumb image url
        large (str): coin large image url

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id": 'id',
        "name": 'name',
        "api_symbol": 'api_symbol',
        "symbol": 'symbol',
        "market_cap_rank": 'market_cap_rank',
        "thumb": 'thumb',
        "large": 'large'
    }

    _optionals = [
        'id',
        'name',
        'api_symbol',
        'symbol',
        'market_cap_rank',
        'thumb',
        'large',
    ]

    def __init__(self,
                 id=APIHelper.SKIP,
                 name=APIHelper.SKIP,
                 api_symbol=APIHelper.SKIP,
                 symbol=APIHelper.SKIP,
                 market_cap_rank=APIHelper.SKIP,
                 thumb=APIHelper.SKIP,
                 large=APIHelper.SKIP):
        """Constructor for the Coin class"""

        # Initialize members of the class
        if id is not APIHelper.SKIP:
            self.id = id 
        if name is not APIHelper.SKIP:
            self.name = name 
        if api_symbol is not APIHelper.SKIP:
            self.api_symbol = api_symbol 
        if symbol is not APIHelper.SKIP:
            self.symbol = symbol 
        if market_cap_rank is not APIHelper.SKIP:
            self.market_cap_rank = market_cap_rank 
        if thumb is not APIHelper.SKIP:
            self.thumb = thumb 
        if large is not APIHelper.SKIP:
            self.large = large 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id") if dictionary.get("id") else APIHelper.SKIP
        name = dictionary.get("name") if dictionary.get("name") else APIHelper.SKIP
        api_symbol = dictionary.get("api_symbol") if dictionary.get("api_symbol") else APIHelper.SKIP
        symbol = dictionary.get("symbol") if dictionary.get("symbol") else APIHelper.SKIP
        market_cap_rank = dictionary.get("market_cap_rank") if dictionary.get("market_cap_rank") else APIHelper.SKIP
        thumb = dictionary.get("thumb") if dictionary.get("thumb") else APIHelper.SKIP
        large = dictionary.get("large") if dictionary.get("large") else APIHelper.SKIP
        # Return an object of this model
        return cls(id,
                   name,
                   api_symbol,
                   symbol,
                   market_cap_rank,
                   thumb,
                   large)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'id={(self.id if hasattr(self, "id") else None)!r}, '
                f'name={(self.name if hasattr(self, "name") else None)!r}, '
                f'api_symbol={(self.api_symbol if hasattr(self, "api_symbol") else None)!r}, '
                f'symbol={(self.symbol if hasattr(self, "symbol") else None)!r}, '
                f'market_cap_rank={(self.market_cap_rank if hasattr(self, "market_cap_rank") else None)!r}, '
                f'thumb={(self.thumb if hasattr(self, "thumb") else None)!r}, '
                f'large={(self.large if hasattr(self, "large") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'id={(self.id if hasattr(self, "id") else None)!s}, '
                f'name={(self.name if hasattr(self, "name") else None)!s}, '
                f'api_symbol={(self.api_symbol if hasattr(self, "api_symbol") else None)!s}, '
                f'symbol={(self.symbol if hasattr(self, "symbol") else None)!s}, '
                f'market_cap_rank={(self.market_cap_rank if hasattr(self, "market_cap_rank") else None)!s}, '
                f'thumb={(self.thumb if hasattr(self, "thumb") else None)!s}, '
                f'large={(self.large if hasattr(self, "large") else None)!s})')



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from apimatic_core_interfaces.types.http_method_enum import HttpMethodEnum


class HttpMethodEnum(HttpMethodEnum):

    """Enumeration of an HTTP Method

    Attributes:
        GET: A GET Request
        POST: A POST Request
        PUT: A PUT Request
        PATCH: A PATCH Request
        DELETE: A DELETE Request

    """


# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from coingeckopublicapiv3.api_helper import APIHelper


class ExchangeData(object):

    """Implementation of the 'ExchangeData' model.

    Attributes:
        name (str): exchange name
        year_established (float): exchange established year
        country (str): exchange incorporated country
        description (str): exchange description
        url (str): exchange website url
        image (str): exchange image url
        facebook_url (str): exchange facebook url
        reddit_url (str): exchange reddit url
        telegram_url (str): exchange telegram url
        slack_url (str): exchange slack url
        other_url_1 (str): The model property of type str.
        other_url_2 (str): The model property of type str.
        twitter_handle (str): exchange twitter handle
        has_trading_incentive (bool): exchange trading incentive
        centralized (bool): exchange type (true for centralized, false for
            decentralized)
        public_notice (str): public notice for exchange
        alert_notice (str): alert notice for exchange
        trust_score (float): exchange trust score
        trust_score_rank (float): exchange trust score rank
        trade_volume_24_h_btc (float): The model property of type float.
        coins (float): number of coins listed on the exchange
        pairs (float): number of trading pairs on the exchange
        tickers (List[Any]): The model property of type List[Any].

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name": 'name',
        "year_established": 'year_established',
        "country": 'country',
        "description": 'description',
        "url": 'url',
        "image": 'image',
        "facebook_url": 'facebook_url',
        "reddit_url": 'reddit_url',
        "telegram_url": 'telegram_url',
        "slack_url": 'slack_url',
        "other_url_1": 'other_url_1',
        "other_url_2": 'other_url_2',
        "twitter_handle": 'twitter_handle',
        "has_trading_incentive": 'has_trading_incentive',
        "centralized": 'centralized',
        "public_notice": 'public_notice',
        "alert_notice": 'alert_notice',
        "trust_score": 'trust_score',
        "trust_score_rank": 'trust_score_rank',
        "trade_volume_24_h_btc": 'trade_volume_24h_btc',
        "coins": 'coins',
        "pairs": 'pairs',
        "tickers": 'tickers'
    }

    _optionals = [
        'name',
        'year_established',
        'country',
        'description',
        'url',
        'image',
        'facebook_url',
        'reddit_url',
        'telegram_url',
        'slack_url',
        'other_url_1',
        'other_url_2',
        'twitter_handle',
        'has_trading_incentive',
        'centralized',
        'public_notice',
        'alert_notice',
        'trust_score',
        'trust_score_rank',
        'trade_volume_24_h_btc',
        'coins',
        'pairs',
        'tickers',
    ]

    def __init__(self,
                 name=APIHelper.SKIP,
                 year_established=APIHelper.SKIP,
                 country=APIHelper.SKIP,
                 description=APIHelper.SKIP,
                 url=APIHelper.SKIP,
                 image=APIHelper.SKIP,
                 facebook_url=APIHelper.SKIP,
                 reddit_url=APIHelper.SKIP,
                 telegram_url=APIHelper.SKIP,
                 slack_url=APIHelper.SKIP,
                 other_url_1=APIHelper.SKIP,
                 other_url_2=APIHelper.SKIP,
                 twitter_handle=APIHelper.SKIP,
                 has_trading_incentive=APIHelper.SKIP,
                 centralized=APIHelper.SKIP,
                 public_notice=APIHelper.SKIP,
                 alert_notice=APIHelper.SKIP,
                 trust_score=APIHelper.SKIP,
                 trust_score_rank=APIHelper.SKIP,
                 trade_volume_24_h_btc=APIHelper.SKIP,
                 coins=APIHelper.SKIP,
                 pairs=APIHelper.SKIP,
                 tickers=APIHelper.SKIP):
        """Constructor for the ExchangeData class"""

        # Initialize members of the class
        if name is not APIHelper.SKIP:
            self.name = name 
        if year_established is not APIHelper.SKIP:
            self.year_established = year_established 
        if country is not APIHelper.SKIP:
            self.country = country 
        if description is not APIHelper.SKIP:
            self.description = description 
        if url is not APIHelper.SKIP:
            self.url = url 
        if image is not APIHelper.SKIP:
            self.image = image 
        if facebook_url is not APIHelper.SKIP:
            self.facebook_url = facebook_url 
        if reddit_url is not APIHelper.SKIP:
            self.reddit_url = reddit_url 
        if telegram_url is not APIHelper.SKIP:
            self.telegram_url = telegram_url 
        if slack_url is not APIHelper.SKIP:
            self.slack_url = slack_url 
        if other_url_1 is not APIHelper.SKIP:
            self.other_url_1 = other_url_1 
        if other_url_2 is not APIHelper.SKIP:
            self.other_url_2 = other_url_2 
        if twitter_handle is not APIHelper.SKIP:
            self.twitter_handle = twitter_handle 
        if has_trading_incentive is not APIHelper.SKIP:
            self.has_trading_incentive = has_trading_incentive 
        if centralized is not APIHelper.SKIP:
            self.centralized = centralized 
        if public_notice is not APIHelper.SKIP:
            self.public_notice = public_notice 
        if alert_notice is not APIHelper.SKIP:
            self.alert_notice = alert_notice 
        if trust_score is not APIHelper.SKIP:
            self.trust_score = trust_score 
        if trust_score_rank is not APIHelper.SKIP:
            self.trust_score_rank = trust_score_rank 
        if trade_volume_24_h_btc is not APIHelper.SKIP:
            self.trade_volume_24_h_btc = trade_volume_24_h_btc 
        if coins is not APIHelper.SKIP:
            self.coins = coins 
        if pairs is not APIHelper.SKIP:
            self.pairs = pairs 
        if tickers is not APIHelper.SKIP:
            self.tickers = tickers 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        name = dictionary.get("name") if dictionary.get("name") else APIHelper.SKIP
        year_established = dictionary.get("year_established") if dictionary.get("year_established") else APIHelper.SKIP
        country = dictionary.get("country") if dictionary.get("country") else APIHelper.SKIP
        description = dictionary.get("description") if dictionary.get("description") else APIHelper.SKIP
        url = dictionary.get("url") if dictionary.get("url") else APIHelper.SKIP
        image = dictionary.get("image") if dictionary.get("image") else APIHelper.SKIP
        facebook_url = dictionary.get("facebook_url") if dictionary.get("facebook_url") else APIHelper.SKIP
        reddit_url = dictionary.get("reddit_url") if dictionary.get("reddit_url") else APIHelper.SKIP
        telegram_url = dictionary.get("telegram_url") if dictionary.get("telegram_url") else APIHelper.SKIP
        slack_url = dictionary.get("slack_url") if dictionary.get("slack_url") else APIHelper.SKIP
        other_url_1 = dictionary.get("other_url_1") if dictionary.get("other_url_1") else APIHelper.SKIP
        other_url_2 = dictionary.get("other_url_2") if dictionary.get("other_url_2") else APIHelper.SKIP
        twitter_handle = dictionary.get("twitter_handle") if dictionary.get("twitter_handle") else APIHelper.SKIP
        has_trading_incentive = dictionary.get("has_trading_incentive") if "has_trading_incentive" in dictionary.keys() else APIHelper.SKIP
        centralized = dictionary.get("centralized") if "centralized" in dictionary.keys() else APIHelper.SKIP
        public_notice = dictionary.get("public_notice") if dictionary.get("public_notice") else APIHelper.SKIP
        alert_notice = dictionary.get("alert_notice") if dictionary.get("alert_notice") else APIHelper.SKIP
        trust_score = dictionary.get("trust_score") if dictionary.get("trust_score") else APIHelper.SKIP
        trust_score_rank = dictionary.get("trust_score_rank") if dictionary.get("trust_score_rank") else APIHelper.SKIP
        trade_volume_24_h_btc = dictionary.get("trade_volume_24h_btc") if dictionary.get("trade_volume_24h_btc") else APIHelper.SKIP
        coins = dictionary.get("coins") if dictionary.get("coins") else APIHelper.SKIP
        pairs = dictionary.get("pairs") if dictionary.get("pairs") else APIHelper.SKIP
        tickers = dictionary.get("tickers") if dictionary.get("tickers") else APIHelper.SKIP
        # Return an object of this model
        return cls(name,
                   year_established,
                   country,
                   description,
                   url,
                   image,
                   facebook_url,
                   reddit_url,
                   telegram_url,
                   slack_url,
                   other_url_1,
                   other_url_2,
                   twitter_handle,
                   has_trading_incentive,
                   centralized,
                   public_notice,
                   alert_notice,
                   trust_score,
                   trust_score_rank,
                   trade_volume_24_h_btc,
                   coins,
                   pairs,
                   tickers)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'name={(self.name if hasattr(self, "name") else None)!r}, '
                f'year_established={(self.year_established if hasattr(self, "year_established") else None)!r}, '
                f'country={(self.country if hasattr(self, "country") else None)!r}, '
                f'description={(self.description if hasattr(self, "description") else None)!r}, '
                f'url={(self.url if hasattr(self, "url") else None)!r}, '
                f'image={(self.image if hasattr(self, "image") else None)!r}, '
                f'facebook_url={(self.facebook_url if hasattr(self, "facebook_url") else None)!r}, '
                f'reddit_url={(self.reddit_url if hasattr(self, "reddit_url") else None)!r}, '
                f'telegram_url={(self.telegram_url if hasattr(self, "telegram_url") else None)!r}, '
                f'slack_url={(self.slack_url if hasattr(self, "slack_url") else None)!r}, '
                f'other_url_1={(self.other_url_1 if hasattr(self, "other_url_1") else None)!r}, '
                f'other_url_2={(self.other_url_2 if hasattr(self, "other_url_2") else None)!r}, '
                f'twitter_handle={(self.twitter_handle if hasattr(self, "twitter_handle") else None)!r}, '
                f'has_trading_incentive={(self.has_trading_incentive if hasattr(self, "has_trading_incentive") else None)!r}, '
                f'centralized={(self.centralized if hasattr(self, "centralized") else None)!r}, '
                f'public_notice={(self.public_notice if hasattr(self, "public_notice") else None)!r}, '
                f'alert_notice={(self.alert_notice if hasattr(self, "alert_notice") else None)!r}, '
                f'trust_score={(self.trust_score if hasattr(self, "trust_score") else None)!r}, '
                f'trust_score_rank={(self.trust_score_rank if hasattr(self, "trust_score_rank") else None)!r}, '
                f'trade_volume_24_h_btc={(self.trade_volume_24_h_btc if hasattr(self, "trade_volume_24_h_btc") else None)!r}, '
                f'coins={(self.coins if hasattr(self, "coins") else None)!r}, '
                f'pairs={(self.pairs if hasattr(self, "pairs") else None)!r}, '
                f'tickers={(self.tickers if hasattr(self, "tickers") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'name={(self.name if hasattr(self, "name") else None)!s}, '
                f'year_established={(self.year_established if hasattr(self, "year_established") else None)!s}, '
                f'country={(self.country if hasattr(self, "country") else None)!s}, '
                f'description={(self.description if hasattr(self, "description") else None)!s}, '
                f'url={(self.url if hasattr(self, "url") else None)!s}, '
                f'image={(self.image if hasattr(self, "image") else None)!s}, '
                f'facebook_url={(self.facebook_url if hasattr(self, "facebook_url") else None)!s}, '
                f'reddit_url={(self.reddit_url if hasattr(self, "reddit_url") else None)!s}, '
                f'telegram_url={(self.telegram_url if hasattr(self, "telegram_url") else None)!s}, '
                f'slack_url={(self.slack_url if hasattr(self, "slack_url") else None)!s}, '
                f'other_url_1={(self.other_url_1 if hasattr(self, "other_url_1") else None)!s}, '
                f'other_url_2={(self.other_url_2 if hasattr(self, "other_url_2") else None)!s}, '
                f'twitter_handle={(self.twitter_handle if hasattr(self, "twitter_handle") else None)!s}, '
                f'has_trading_incentive={(self.has_trading_incentive if hasattr(self, "has_trading_incentive") else None)!s}, '
                f'centralized={(self.centralized if hasattr(self, "centralized") else None)!s}, '
                f'public_notice={(self.public_notice if hasattr(self, "public_notice") else None)!s}, '
                f'alert_notice={(self.alert_notice if hasattr(self, "alert_notice") else None)!s}, '
                f'trust_score={(self.trust_score if hasattr(self, "trust_score") else None)!s}, '
                f'trust_score_rank={(self.trust_score_rank if hasattr(self, "trust_score_rank") else None)!s}, '
                f'trade_volume_24_h_btc={(self.trade_volume_24_h_btc if hasattr(self, "trade_volume_24_h_btc") else None)!s}, '
                f'coins={(self.coins if hasattr(self, "coins") else None)!s}, '
                f'pairs={(self.pairs if hasattr(self, "pairs") else None)!s}, '
                f'tickers={(self.tickers if hasattr(self, "tickers") else None)!s})')



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from apimatic_core.authentication.query_auth import QueryAuth


class ApiKeyQueryParam(QueryAuth):

    @property
    def error_message(self):
        """Display error message on occurrence of authentication failure
        in ApiKeyQueryParam

        """
        return "ApiKeyQueryParam: x_cg_demo_api_key is undefined."

    def __init__(self, api_key_query_param_credentials):
        auth_params = {}
        if api_key_query_param_credentials is not None \
                and api_key_query_param_credentials.x_cg_demo_api_key is not None:
            auth_params["x_cg_demo_api_key"] = api_key_query_param_credentials.x_cg_demo_api_key
        super().__init__(auth_params=auth_params)


class ApiKeyQueryParamCredentials:

    @property
    def x_cg_demo_api_key(self):
        return self._x_cg_demo_api_key

    def __init__(self, x_cg_demo_api_key):
        if x_cg_demo_api_key is None:
            raise ValueError('x_cg_demo_api_key cannot be None')
        self._x_cg_demo_api_key = x_cg_demo_api_key

    def clone_with(self, x_cg_demo_api_key=None):
        return ApiKeyQueryParamCredentials(
            x_cg_demo_api_key or self.x_cg_demo_api_key)



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class LocaleEnum(object):

    """Implementation of the 'locale' enum.

    Attributes:
        AR: The enum member of type str.
        BG: The enum member of type str.
        CS: The enum member of type str.
        DA: The enum member of type str.
        DE: The enum member of type str.
        EL: The enum member of type str.
        EN: The enum member of type str.
        ES: The enum member of type str.
        FI: The enum member of type str.
        FR: The enum member of type str.
        HE: The enum member of type str.
        HI: The enum member of type str.
        HR: The enum member of type str.
        HU: The enum member of type str.
        ID: The enum member of type str.
        IT: The enum member of type str.
        JA: The enum member of type str.
        KO: The enum member of type str.
        LT: The enum member of type str.
        NL: The enum member of type str.
        NO: The enum member of type str.
        PL: The enum member of type str.
        PT: The enum member of type str.
        RO: The enum member of type str.
        RU: The enum member of type str.
        SK: The enum member of type str.
        SL: The enum member of type str.
        SV: The enum member of type str.
        TH: The enum member of type str.
        TR: The enum member of type str.
        UK: The enum member of type str.
        VI: The enum member of type str.
        ZH: The enum member of type str.
        ZHTW: The enum member of type str.

    """
    AR = 'ar'

    BG = 'bg'

    CS = 'cs'

    DA = 'da'

    DE = 'de'

    EL = 'el'

    EN = 'en'

    ES = 'es'

    FI = 'fi'

    FR = 'fr'

    HE = 'he'

    HI = 'hi'

    HR = 'hr'

    HU = 'hu'

    ID = 'id'

    IT = 'it'

    JA = 'ja'

    KO = 'ko'

    LT = 'lt'

    NL = 'nl'

    NO = 'no'

    PL = 'pl'

    PT = 'pt'

    RO = 'ro'

    RU = 'ru'

    SK = 'sk'

    SL = 'sl'

    SV = 'sv'

    TH = 'th'

    TR = 'tr'

    UK = 'uk'

    VI = 'vi'

    ZH = 'zh'

    ZHTW = 'zh-tw'




# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from coingeckopublicapiv3.api_helper import APIHelper
from coingeckopublicapiv3.models.current_price_1 import CurrentPrice1
from coingeckopublicapiv3.models.market_cap_1 import MarketCap1
from coingeckopublicapiv3.models.total_volume_1 import TotalVolume1


class MarketData1(object):

    """Implementation of the 'MarketData1' model.

    coin market data

    Attributes:
        current_price (CurrentPrice1): coin current price
        market_cap (MarketCap1): coin market cap
        total_volume (TotalVolume1): coin total volume

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "current_price": 'current_price',
        "market_cap": 'market_cap',
        "total_volume": 'total_volume'
    }

    _optionals = [
        'current_price',
        'market_cap',
        'total_volume',
    ]

    def __init__(self,
                 current_price=APIHelper.SKIP,
                 market_cap=APIHelper.SKIP,
                 total_volume=APIHelper.SKIP):
        """Constructor for the MarketData1 class"""

        # Initialize members of the class
        if current_price is not APIHelper.SKIP:
            self.current_price = current_price 
        if market_cap is not APIHelper.SKIP:
            self.market_cap = market_cap 
        if total_volume is not APIHelper.SKIP:
            self.total_volume = total_volume 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        current_price = CurrentPrice1.from_dictionary(dictionary.get('current_price')) if 'current_price' in dictionary.keys() else APIHelper.SKIP
        market_cap = MarketCap1.from_dictionary(dictionary.get('market_cap')) if 'market_cap' in dictionary.keys() else APIHelper.SKIP
        total_volume = TotalVolume1.from_dictionary(dictionary.get('total_volume')) if 'total_volume' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(current_price,
                   market_cap,
                   total_volume)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'current_price={(self.current_price if hasattr(self, "current_price") else None)!r}, '
                f'market_cap={(self.market_cap if hasattr(self, "market_cap") else None)!r}, '
                f'total_volume={(self.total_volume if hasattr(self, "total_volume") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'current_price={(self.current_price if hasattr(self, "current_price") else None)!s}, '
                f'market_cap={(self.market_cap if hasattr(self, "market_cap") else None)!s}, '
                f'total_volume={(self.total_volume if hasattr(self, "total_volume") else None)!s})')



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from coingeckopublicapiv3.api_helper import APIHelper
from coingeckopublicapiv3.models.derivatives_tickers_list import DerivativesTickersList


class DerivativesExchangesID(object):

    """Implementation of the 'DerivativesExchangesID' model.

    Attributes:
        name (str): derivatives exchange name
        open_interest_btc (float): derivatives exchange open interest in BTC
        trade_volume_24_h_btc (str): derivatives exchange trade volume in BTC
            in 24 hours
        number_of_perpetual_pairs (float): number of perpetual pairs in the
            derivatives exchange
        number_of_futures_pairs (float): number of futures pairs in the
            derivatives exchange
        image (str): derivatives exchange image url
        year_established (float): derivatives exchange established year
        country (str): derivatives exchange incorporated country
        description (str): derivatives exchange description
        url (str): derivatives exchange website url
        tickers (List[DerivativesTickersList]): The model property of type
            List[DerivativesTickersList].

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name": 'name',
        "open_interest_btc": 'open_interest_btc',
        "trade_volume_24_h_btc": 'trade_volume_24h_btc',
        "number_of_perpetual_pairs": 'number_of_perpetual_pairs',
        "number_of_futures_pairs": 'number_of_futures_pairs',
        "image": 'image',
        "year_established": 'year_established',
        "country": 'country',
        "description": 'description',
        "url": 'url',
        "tickers": 'tickers'
    }

    _optionals = [
        'name',
        'open_interest_btc',
        'trade_volume_24_h_btc',
        'number_of_perpetual_pairs',
        'number_of_futures_pairs',
        'image',
        'year_established',
        'country',
        'description',
        'url',
        'tickers',
    ]

    _nullables = [
        'year_established',
        'country',
    ]

    def __init__(self,
                 name=APIHelper.SKIP,
                 open_interest_btc=APIHelper.SKIP,
                 trade_volume_24_h_btc=APIHelper.SKIP,
                 number_of_perpetual_pairs=APIHelper.SKIP,
                 number_of_futures_pairs=APIHelper.SKIP,
                 image=APIHelper.SKIP,
                 year_established=APIHelper.SKIP,
                 country=APIHelper.SKIP,
                 description=APIHelper.SKIP,
                 url=APIHelper.SKIP,
                 tickers=APIHelper.SKIP):
        """Constructor for the DerivativesExchangesID class"""

        # Initialize members of the class
        if name is not APIHelper.SKIP:
            self.name = name 
        if open_interest_btc is not APIHelper.SKIP:
            self.open_interest_btc = open_interest_btc 
        if trade_volume_24_h_btc is not APIHelper.SKIP:
            self.trade_volume_24_h_btc = trade_volume_24_h_btc 
        if number_of_perpetual_pairs is not APIHelper.SKIP:
            self.number_of_perpetual_pairs = number_of_perpetual_pairs 
        if number_of_futures_pairs is not APIHelper.SKIP:
            self.number_of_futures_pairs = number_of_futures_pairs 
        if image is not APIHelper.SKIP:
            self.image = image 
        if year_established is not APIHelper.SKIP:
            self.year_established = year_established 
        if country is not APIHelper.SKIP:
            self.country = country 
        if description is not APIHelper.SKIP:
            self.description = description 
        if url is not APIHelper.SKIP:
            self.url = url 
        if tickers is not APIHelper.SKIP:
            self.tickers = tickers 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        name = dictionary.get("name") if dictionary.get("name") else APIHelper.SKIP
        open_interest_btc = dictionary.get("open_interest_btc") if dictionary.get("open_interest_btc") else APIHelper.SKIP
        trade_volume_24_h_btc = dictionary.get("trade_volume_24h_btc") if dictionary.get("trade_volume_24h_btc") else APIHelper.SKIP
        number_of_perpetual_pairs = dictionary.get("number_of_perpetual_pairs") if dictionary.get("number_of_perpetual_pairs") else APIHelper.SKIP
        number_of_futures_pairs = dictionary.get("number_of_futures_pairs") if dictionary.get("number_of_futures_pairs") else APIHelper.SKIP
        image = dictionary.get("image") if dictionary.get("image") else APIHelper.SKIP
        year_established = dictionary.get("year_established") if "year_established" in dictionary.keys() else APIHelper.SKIP
        country = dictionary.get("country") if "country" in dictionary.keys() else APIHelper.SKIP
        description = dictionary.get("description") if dictionary.get("description") else APIHelper.SKIP
        url = dictionary.get("url") if dictionary.get("url") else APIHelper.SKIP
        tickers = None
        if dictionary.get('tickers') is not None:
            tickers = [DerivativesTickersList.from_dictionary(x) for x in dictionary.get('tickers')]
        else:
            tickers = APIHelper.SKIP
        # Return an object of this model
        return cls(name,
                   open_interest_btc,
                   trade_volume_24_h_btc,
                   number_of_perpetual_pairs,
                   number_of_futures_pairs,
                   image,
                   year_established,
                   country,
                   description,
                   url,
                   tickers)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'name={(self.name if hasattr(self, "name") else None)!r}, '
                f'open_interest_btc={(self.open_interest_btc if hasattr(self, "open_interest_btc") else None)!r}, '
                f'trade_volume_24_h_btc={(self.trade_volume_24_h_btc if hasattr(self, "trade_volume_24_h_btc") else None)!r}, '
                f'number_of_perpetual_pairs={(self.number_of_perpetual_pairs if hasattr(self, "number_of_perpetual_pairs") else None)!r}, '
                f'number_of_futures_pairs={(self.number_of_futures_pairs if hasattr(self, "number_of_futures_pairs") else None)!r}, '
                f'image={(self.image if hasattr(self, "image") else None)!r}, '
                f'year_established={(self.year_established if hasattr(self, "year_established") else None)!r}, '
                f'country={(self.country if hasattr(self, "country") else None)!r}, '
                f'description={(self.description if hasattr(self, "description") else None)!r}, '
                f'url={(self.url if hasattr(self, "url") else None)!r}, '
                f'tickers={(self.tickers if hasattr(self, "tickers") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'name={(self.name if hasattr(self, "name") else None)!s}, '
                f'open_interest_btc={(self.open_interest_btc if hasattr(self, "open_interest_btc") else None)!s}, '
                f'trade_volume_24_h_btc={(self.trade_volume_24_h_btc if hasattr(self, "trade_volume_24_h_btc") else None)!s}, '
                f'number_of_perpetual_pairs={(self.number_of_perpetual_pairs if hasattr(self, "number_of_perpetual_pairs") else None)!s}, '
                f'number_of_futures_pairs={(self.number_of_futures_pairs if hasattr(self, "number_of_futures_pairs") else None)!s}, '
                f'image={(self.image if hasattr(self, "image") else None)!s}, '
                f'year_established={(self.year_established if hasattr(self, "year_established") else None)!s}, '
                f'country={(self.country if hasattr(self, "country") else None)!s}, '
                f'description={(self.description if hasattr(self, "description") else None)!s}, '
                f'url={(self.url if hasattr(self, "url") else None)!s}, '
                f'tickers={(self.tickers if hasattr(self, "tickers") else None)!s})')



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from coingeckopublicapiv3.api_helper import APIHelper
from coingeckopublicapiv3.models.ath import Ath
from coingeckopublicapiv3.models.ath_change_percentage import AthChangePercentage
from coingeckopublicapiv3.models.ath_date import AthDate
from coingeckopublicapiv3.models.atl import Atl
from coingeckopublicapiv3.models.atl_change_percentage import AtlChangePercentage
from coingeckopublicapiv3.models.atl_date import AtlDate
from coingeckopublicapiv3.models.current_price import CurrentPrice
from coingeckopublicapiv3.models.fully_diluted_valuation import FullyDilutedValuation
from coingeckopublicapiv3.models.high_24_h import High24h
from coingeckopublicapiv3.models.low_24_h import Low24h
from coingeckopublicapiv3.models.market_cap import MarketCap
from coingeckopublicapiv3.models.market_cap_change_24_h_in_currency import MarketCapChange24hInCurrency
from coingeckopublicapiv3.models.market_cap_change_percentage_24_h_in_currency import MarketCapChangePercentage24hInCurrency
from coingeckopublicapiv3.models.price_change_percentage_1_h_in_currency import PriceChangePercentage1hInCurrency
from coingeckopublicapiv3.models.price_change_percentage_1_y_in_currency import PriceChangePercentage1yInCurrency
from coingeckopublicapiv3.models.price_change_percentage_14_d_in_currency import PriceChangePercentage14dInCurrency
from coingeckopublicapiv3.models.price_change_percentage_200_d_in_currency import PriceChangePercentage200dInCurrency
from coingeckopublicapiv3.models.price_change_percentage_24_h_in_currency import PriceChangePercentage24hInCurrency
from coingeckopublicapiv3.models.price_change_percentage_30_d_in_currency import PriceChangePercentage30dInCurrency
from coingeckopublicapiv3.models.price_change_percentage_60_d_in_currency import PriceChangePercentage60dInCurrency
from coingeckopublicapiv3.models.price_change_percentage_7_d_in_currency import PriceChangePercentage7dInCurrency
from coingeckopublicapiv3.models.total_volume import TotalVolume


class MarketData(object):

    """Implementation of the 'MarketData' model.

    coin market data

    Attributes:
        current_price (CurrentPrice): coin current price in currency
        total_value_locked (float): total value locked
        mcap_to_tvl_ratio (float): market cap to total value locked ratio
        fdv_to_tvl_ratio (float): fully diluted valuation to total value
            locked ratio
        roi (float): coin return on investment
        ath (Ath): coin all time high (ATH) in currency
        ath_change_percentage (AthChangePercentage): coin all time high (ATH)
            change in percentage
        ath_date (AthDate): coin all time high (ATH) date
        atl (Atl): coin all time low (atl) in currency
        atl_change_percentage (AtlChangePercentage): coin all time low (atl)
            change in percentage
        atl_date (AtlDate): coin all time low (atl) date
        market_cap (MarketCap): coin market cap in currency
        market_cap_rank (float): coin rank by market cap
        fully_diluted_valuation (FullyDilutedValuation): coin fully diluted
            valuation (fdv) in currency
        market_cap_fdv_ratio (float): market cap to fully diluted valuation
            ratio
        total_volume (TotalVolume): coin total trading volume in currency
        high_24_h (High24h): coin 24hr price high in currency
        low_24_h (Low24h): coin 24hr price low in currency
        price_change_24_h (float): coin 24hr price change in currency
        price_change_percentage_24_h (float): coin 24hr price change in
            percentage
        price_change_percentage_7_d (float): coin 7d price change in percentage
        price_change_percentage_14_d (float): coin 14d price change in
            percentage
        price_change_percentage_30_d (float): coin 30d price change in
            percentage
        price_change_percentage_60_d (float): coin 60d price change in
            percentage
        price_change_percentage_200_d (float): coin 200d price change in
            percentage
        price_change_percentage_1_y (float): coin 1y price change in percentage
        market_cap_change_24_h (float): coin 24hr market cap change in currency
        market_cap_change_percentage_24_h (float): coin 24hr market cap change
            in percentage
        price_change_percentage_1_h_in_currency
            (PriceChangePercentage1hInCurrency): coin 1h price change in
            currency
        price_change_percentage_24_h_in_currency
            (PriceChangePercentage24hInCurrency): coin 24hr price change in
            currency
        price_change_percentage_7_d_in_currency
            (PriceChangePercentage7dInCurrency): coin 7d price change in
            currency
        price_change_percentage_14_d_in_currency
            (PriceChangePercentage14dInCurrency): coin 14d price change in
            currency
        price_change_percentage_30_d_in_currency
            (PriceChangePercentage30dInCurrency): coin 30d price change in
            currency
        price_change_percentage_60_d_in_currency
            (PriceChangePercentage60dInCurrency): coin 60d price change in
            currency
        price_change_percentage_200_d_in_currency
            (PriceChangePercentage200dInCurrency): coin 200d price change in
            currency
        price_change_percentage_1_y_in_currency
            (PriceChangePercentage1yInCurrency): coin 1y price change in
            currency
        market_cap_change_24_h_in_currency (MarketCapChange24hInCurrency):
            coin 24hr market cap change in currency
        market_cap_change_percentage_24_h_in_currency
            (MarketCapChangePercentage24hInCurrency): coin 24hr market cap
            change in percentage
        total_supply (float): coin total supply
        max_supply (float): coin max supply
        circulating_supply (float): coin circulating supply
        last_updated (datetime): coin market data last updated timestamp

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "current_price": 'current_price',
        "total_value_locked": 'total_value_locked',
        "mcap_to_tvl_ratio": 'mcap_to_tvl_ratio',
        "fdv_to_tvl_ratio": 'fdv_to_tvl_ratio',
        "roi": 'roi',
        "ath": 'ath',
        "ath_change_percentage": 'ath_change_percentage',
        "ath_date": 'ath_date',
        "atl": 'atl',
        "atl_change_percentage": 'atl_change_percentage',
        "atl_date": 'atl_date',
        "market_cap": 'market_cap',
        "market_cap_rank": 'market_cap_rank',
        "fully_diluted_valuation": 'fully_diluted_valuation',
        "market_cap_fdv_ratio": 'market_cap_fdv_ratio',
        "total_volume": 'total_volume',
        "high_24_h": 'high_24h',
        "low_24_h": 'low_24h',
        "price_change_24_h": 'price_change_24h',
        "price_change_percentage_24_h": 'price_change_percentage_24h',
        "price_change_percentage_7_d": 'price_change_percentage_7d',
        "price_change_percentage_14_d": 'price_change_percentage_14d',
        "price_change_percentage_30_d": 'price_change_percentage_30d',
        "price_change_percentage_60_d": 'price_change_percentage_60d',
        "price_change_percentage_200_d": 'price_change_percentage_200d',
        "price_change_percentage_1_y": 'price_change_percentage_1y',
        "market_cap_change_24_h": 'market_cap_change_24h',
        "market_cap_change_percentage_24_h": 'market_cap_change_percentage_24h',
        "price_change_percentage_1_h_in_currency": 'price_change_percentage_1h_in_currency',
        "price_change_percentage_24_h_in_currency": 'price_change_percentage_24h_in_currency',
        "price_change_percentage_7_d_in_currency": 'price_change_percentage_7d_in_currency',
        "price_change_percentage_14_d_in_currency": 'price_change_percentage_14d_in_currency',
        "price_change_percentage_30_d_in_currency": 'price_change_percentage_30d_in_currency',
        "price_change_percentage_60_d_in_currency": 'price_change_percentage_60d_in_currency',
        "price_change_percentage_200_d_in_currency": 'price_change_percentage_200d_in_currency',
        "price_change_percentage_1_y_in_currency": 'price_change_percentage_1y_in_currency',
        "market_cap_change_24_h_in_currency": 'market_cap_change_24h_in_currency',
        "market_cap_change_percentage_24_h_in_currency": 'market_cap_change_percentage_24h_in_currency',
        "total_supply": 'total_supply',
        "max_supply": 'max_supply',
        "circulating_supply": 'circulating_supply',
        "last_updated": 'last_updated'
    }

    _optionals = [
        'current_price',
        'total_value_locked',
        'mcap_to_tvl_ratio',
        'fdv_to_tvl_ratio',
        'roi',
        'ath',
        'ath_change_percentage',
        'ath_date',
        'atl',
        'atl_change_percentage',
        'atl_date',
        'market_cap',
        'market_cap_rank',
        'fully_diluted_valuation',
        'market_cap_fdv_ratio',
        'total_volume',
        'high_24_h',
        'low_24_h',
        'price_change_24_h',
        'price_change_percentage_24_h',
        'price_change_percentage_7_d',
        'price_change_percentage_14_d',
        'price_change_percentage_30_d',
        'price_change_percentage_60_d',
        'price_change_percentage_200_d',
        'price_change_percentage_1_y',
        'market_cap_change_24_h',
        'market_cap_change_percentage_24_h',
        'price_change_percentage_1_h_in_currency',
        'price_change_percentage_24_h_in_currency',
        'price_change_percentage_7_d_in_currency',
        'price_change_percentage_14_d_in_currency',
        'price_change_percentage_30_d_in_currency',
        'price_change_percentage_60_d_in_currency',
        'price_change_percentage_200_d_in_currency',
        'price_change_percentage_1_y_in_currency',
        'market_cap_change_24_h_in_currency',
        'market_cap_change_percentage_24_h_in_currency',
        'total_supply',
        'max_supply',
        'circulating_supply',
        'last_updated',
    ]

    def __init__(self,
                 current_price=APIHelper.SKIP,
                 total_value_locked=APIHelper.SKIP,
                 mcap_to_tvl_ratio=APIHelper.SKIP,
                 fdv_to_tvl_ratio=APIHelper.SKIP,
                 roi=APIHelper.SKIP,
                 ath=APIHelper.SKIP,
                 ath_change_percentage=APIHelper.SKIP,
                 ath_date=APIHelper.SKIP,
                 atl=APIHelper.SKIP,
                 atl_change_percentage=APIHelper.SKIP,
                 atl_date=APIHelper.SKIP,
                 market_cap=APIHelper.SKIP,
                 market_cap_rank=APIHelper.SKIP,
                 fully_diluted_valuation=APIHelper.SKIP,
                 market_cap_fdv_ratio=APIHelper.SKIP,
                 total_volume=APIHelper.SKIP,
                 high_24_h=APIHelper.SKIP,
                 low_24_h=APIHelper.SKIP,
                 price_change_24_h=APIHelper.SKIP,
                 price_change_percentage_24_h=APIHelper.SKIP,
                 price_change_percentage_7_d=APIHelper.SKIP,
                 price_change_percentage_14_d=APIHelper.SKIP,
                 price_change_percentage_30_d=APIHelper.SKIP,
                 price_change_percentage_60_d=APIHelper.SKIP,
                 price_change_percentage_200_d=APIHelper.SKIP,
                 price_change_percentage_1_y=APIHelper.SKIP,
                 market_cap_change_24_h=APIHelper.SKIP,
                 market_cap_change_percentage_24_h=APIHelper.SKIP,
                 price_change_percentage_1_h_in_currency=APIHelper.SKIP,
                 price_change_percentage_24_h_in_currency=APIHelper.SKIP,
                 price_change_percentage_7_d_in_currency=APIHelper.SKIP,
                 price_change_percentage_14_d_in_currency=APIHelper.SKIP,
                 price_change_percentage_30_d_in_currency=APIHelper.SKIP,
                 price_change_percentage_60_d_in_currency=APIHelper.SKIP,
                 price_change_percentage_200_d_in_currency=APIHelper.SKIP,
                 price_change_percentage_1_y_in_currency=APIHelper.SKIP,
                 market_cap_change_24_h_in_currency=APIHelper.SKIP,
                 market_cap_change_percentage_24_h_in_currency=APIHelper.SKIP,
                 total_supply=APIHelper.SKIP,
                 max_supply=APIHelper.SKIP,
                 circulating_supply=APIHelper.SKIP,
                 last_updated=APIHelper.SKIP):
        """Constructor for the MarketData class"""

        # Initialize members of the class
        if current_price is not APIHelper.SKIP:
            self.current_price = current_price 
        if total_value_locked is not APIHelper.SKIP:
            self.total_value_locked = total_value_locked 
        if mcap_to_tvl_ratio is not APIHelper.SKIP:
            self.mcap_to_tvl_ratio = mcap_to_tvl_ratio 
        if fdv_to_tvl_ratio is not APIHelper.SKIP:
            self.fdv_to_tvl_ratio = fdv_to_tvl_ratio 
        if roi is not APIHelper.SKIP:
            self.roi = roi 
        if ath is not APIHelper.SKIP:
            self.ath = ath 
        if ath_change_percentage is not APIHelper.SKIP:
            self.ath_change_percentage = ath_change_percentage 
        if ath_date is not APIHelper.SKIP:
            self.ath_date = ath_date 
        if atl is not APIHelper.SKIP:
            self.atl = atl 
        if atl_change_percentage is not APIHelper.SKIP:
            self.atl_change_percentage = atl_change_percentage 
        if atl_date is not APIHelper.SKIP:
            self.atl_date = atl_date 
        if market_cap is not APIHelper.SKIP:
            self.market_cap = market_cap 
        if market_cap_rank is not APIHelper.SKIP:
            self.market_cap_rank = market_cap_rank 
        if fully_diluted_valuation is not APIHelper.SKIP:
            self.fully_diluted_valuation = fully_diluted_valuation 
        if market_cap_fdv_ratio is not APIHelper.SKIP:
            self.market_cap_fdv_ratio = market_cap_fdv_ratio 
        if total_volume is not APIHelper.SKIP:
            self.total_volume = total_volume 
        if high_24_h is not APIHelper.SKIP:
            self.high_24_h = high_24_h 
        if low_24_h is not APIHelper.SKIP:
            self.low_24_h = low_24_h 
        if price_change_24_h is not APIHelper.SKIP:
            self.price_change_24_h = price_change_24_h 
        if price_change_percentage_24_h is not APIHelper.SKIP:
            self.price_change_percentage_24_h = price_change_percentage_24_h 
        if price_change_percentage_7_d is not APIHelper.SKIP:
            self.price_change_percentage_7_d = price_change_percentage_7_d 
        if price_change_percentage_14_d is not APIHelper.SKIP:
            self.price_change_percentage_14_d = price_change_percentage_14_d 
        if price_change_percentage_30_d is not APIHelper.SKIP:
            self.price_change_percentage_30_d = price_change_percentage_30_d 
        if price_change_percentage_60_d is not APIHelper.SKIP:
            self.price_change_percentage_60_d = price_change_percentage_60_d 
        if price_change_percentage_200_d is not APIHelper.SKIP:
            self.price_change_percentage_200_d = price_change_percentage_200_d 
        if price_change_percentage_1_y is not APIHelper.SKIP:
            self.price_change_percentage_1_y = price_change_percentage_1_y 
        if market_cap_change_24_h is not APIHelper.SKIP:
            self.market_cap_change_24_h = market_cap_change_24_h 
        if market_cap_change_percentage_24_h is not APIHelper.SKIP:
            self.market_cap_change_percentage_24_h = market_cap_change_percentage_24_h 
        if price_change_percentage_1_h_in_currency is not APIHelper.SKIP:
            self.price_change_percentage_1_h_in_currency = price_change_percentage_1_h_in_currency 
        if price_change_percentage_24_h_in_currency is not APIHelper.SKIP:
            self.price_change_percentage_24_h_in_currency = price_change_percentage_24_h_in_currency 
        if price_change_percentage_7_d_in_currency is not APIHelper.SKIP:
            self.price_change_percentage_7_d_in_currency = price_change_percentage_7_d_in_currency 
        if price_change_percentage_14_d_in_currency is not APIHelper.SKIP:
            self.price_change_percentage_14_d_in_currency = price_change_percentage_14_d_in_currency 
        if price_change_percentage_30_d_in_currency is not APIHelper.SKIP:
            self.price_change_percentage_30_d_in_currency = price_change_percentage_30_d_in_currency 
        if price_change_percentage_60_d_in_currency is not APIHelper.SKIP:
            self.price_change_percentage_60_d_in_currency = price_change_percentage_60_d_in_currency 
        if price_change_percentage_200_d_in_currency is not APIHelper.SKIP:
            self.price_change_percentage_200_d_in_currency = price_change_percentage_200_d_in_currency 
        if price_change_percentage_1_y_in_currency is not APIHelper.SKIP:
            self.price_change_percentage_1_y_in_currency = price_change_percentage_1_y_in_currency 
        if market_cap_change_24_h_in_currency is not APIHelper.SKIP:
            self.market_cap_change_24_h_in_currency = market_cap_change_24_h_in_currency 
        if market_cap_change_percentage_24_h_in_currency is not APIHelper.SKIP:
            self.market_cap_change_percentage_24_h_in_currency = market_cap_change_percentage_24_h_in_currency 
        if total_supply is not APIHelper.SKIP:
            self.total_supply = total_supply 
        if max_supply is not APIHelper.SKIP:
            self.max_supply = max_supply 
        if circulating_supply is not APIHelper.SKIP:
            self.circulating_supply = circulating_supply 
        if last_updated is not APIHelper.SKIP:
            self.last_updated = APIHelper.apply_datetime_converter(last_updated, APIHelper.RFC3339DateTime) if last_updated else None 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        current_price = CurrentPrice.from_dictionary(dictionary.get('current_price')) if 'current_price' in dictionary.keys() else APIHelper.SKIP
        total_value_locked = dictionary.get("total_value_locked") if dictionary.get("total_value_locked") else APIHelper.SKIP
        mcap_to_tvl_ratio = dictionary.get("mcap_to_tvl_ratio") if dictionary.get("mcap_to_tvl_ratio") else APIHelper.SKIP
        fdv_to_tvl_ratio = dictionary.get("fdv_to_tvl_ratio") if dictionary.get("fdv_to_tvl_ratio") else APIHelper.SKIP
        roi = dictionary.get("roi") if dictionary.get("roi") else APIHelper.SKIP
        ath = Ath.from_dictionary(dictionary.get('ath')) if 'ath' in dictionary.keys() else APIHelper.SKIP
        ath_change_percentage = AthChangePercentage.from_dictionary(dictionary.get('ath_change_percentage')) if 'ath_change_percentage' in dictionary.keys() else APIHelper.SKIP
        ath_date = AthDate.from_dictionary(dictionary.get('ath_date')) if 'ath_date' in dictionary.keys() else APIHelper.SKIP
        atl = Atl.from_dictionary(dictionary.get('atl')) if 'atl' in dictionary.keys() else APIHelper.SKIP
        atl_change_percentage = AtlChangePercentage.from_dictionary(dictionary.get('atl_change_percentage')) if 'atl_change_percentage' in dictionary.keys() else APIHelper.SKIP
        atl_date = AtlDate.from_dictionary(dictionary.get('atl_date')) if 'atl_date' in dictionary.keys() else APIHelper.SKIP
        market_cap = MarketCap.from_dictionary(dictionary.get('market_cap')) if 'market_cap' in dictionary.keys() else APIHelper.SKIP
        market_cap_rank = dictionary.get("market_cap_rank") if dictionary.get("market_cap_rank") else APIHelper.SKIP
        fully_diluted_valuation = FullyDilutedValuation.from_dictionary(dictionary.get('fully_diluted_valuation')) if 'fully_diluted_valuation' in dictionary.keys() else APIHelper.SKIP
        market_cap_fdv_ratio = dictionary.get("market_cap_fdv_ratio") if dictionary.get("market_cap_fdv_ratio") else APIHelper.SKIP
        total_volume = TotalVolume.from_dictionary(dictionary.get('total_volume')) if 'total_volume' in dictionary.keys() else APIHelper.SKIP
        high_24_h = High24h.from_dictionary(dictionary.get('high_24h')) if 'high_24h' in dictionary.keys() else APIHelper.SKIP
        low_24_h = Low24h.from_dictionary(dictionary.get('low_24h')) if 'low_24h' in dictionary.keys() else APIHelper.SKIP
        price_change_24_h = dictionary.get("price_change_24h") if dictionary.get("price_change_24h") else APIHelper.SKIP
        price_change_percentage_24_h = dictionary.get("price_change_percentage_24h") if dictionary.get("price_change_percentage_24h") else APIHelper.SKIP
        price_change_percentage_7_d = dictionary.get("price_change_percentage_7d") if dictionary.get("price_change_percentage_7d") else APIHelper.SKIP
        price_change_percentage_14_d = dictionary.get("price_change_percentage_14d") if dictionary.get("price_change_percentage_14d") else APIHelper.SKIP
        price_change_percentage_30_d = dictionary.get("price_change_percentage_30d") if dictionary.get("price_change_percentage_30d") else APIHelper.SKIP
        price_change_percentage_60_d = dictionary.get("price_change_percentage_60d") if dictionary.get("price_change_percentage_60d") else APIHelper.SKIP
        price_change_percentage_200_d = dictionary.get("price_change_percentage_200d") if dictionary.get("price_change_percentage_200d") else APIHelper.SKIP
        price_change_percentage_1_y = dictionary.get("price_change_percentage_1y") if dictionary.get("price_change_percentage_1y") else APIHelper.SKIP
        market_cap_change_24_h = dictionary.get("market_cap_change_24h") if dictionary.get("market_cap_change_24h") else APIHelper.SKIP
        market_cap_change_percentage_24_h = dictionary.get("market_cap_change_percentage_24h") if dictionary.get("market_cap_change_percentage_24h") else APIHelper.SKIP
        price_change_percentage_1_h_in_currency = PriceChangePercentage1hInCurrency.from_dictionary(dictionary.get('price_change_percentage_1h_in_currency')) if 'price_change_percentage_1h_in_currency' in dictionary.keys() else APIHelper.SKIP
        price_change_percentage_24_h_in_currency = PriceChangePercentage24hInCurrency.from_dictionary(dictionary.get('price_change_percentage_24h_in_currency')) if 'price_change_percentage_24h_in_currency' in dictionary.keys() else APIHelper.SKIP
        price_change_percentage_7_d_in_currency = PriceChangePercentage7dInCurrency.from_dictionary(dictionary.get('price_change_percentage_7d_in_currency')) if 'price_change_percentage_7d_in_currency' in dictionary.keys() else APIHelper.SKIP
        price_change_percentage_14_d_in_currency = PriceChangePercentage14dInCurrency.from_dictionary(dictionary.get('price_change_percentage_14d_in_currency')) if 'price_change_percentage_14d_in_currency' in dictionary.keys() else APIHelper.SKIP
        price_change_percentage_30_d_in_currency = PriceChangePercentage30dInCurrency.from_dictionary(dictionary.get('price_change_percentage_30d_in_currency')) if 'price_change_percentage_30d_in_currency' in dictionary.keys() else APIHelper.SKIP
        price_change_percentage_60_d_in_currency = PriceChangePercentage60dInCurrency.from_dictionary(dictionary.get('price_change_percentage_60d_in_currency')) if 'price_change_percentage_60d_in_currency' in dictionary.keys() else APIHelper.SKIP
        price_change_percentage_200_d_in_currency = PriceChangePercentage200dInCurrency.from_dictionary(dictionary.get('price_change_percentage_200d_in_currency')) if 'price_change_percentage_200d_in_currency' in dictionary.keys() else APIHelper.SKIP
        price_change_percentage_1_y_in_currency = PriceChangePercentage1yInCurrency.from_dictionary(dictionary.get('price_change_percentage_1y_in_currency')) if 'price_change_percentage_1y_in_currency' in dictionary.keys() else APIHelper.SKIP
        market_cap_change_24_h_in_currency = MarketCapChange24hInCurrency.from_dictionary(dictionary.get('market_cap_change_24h_in_currency')) if 'market_cap_change_24h_in_currency' in dictionary.keys() else APIHelper.SKIP
        market_cap_change_percentage_24_h_in_currency = MarketCapChangePercentage24hInCurrency.from_dictionary(dictionary.get('market_cap_change_percentage_24h_in_currency')) if 'market_cap_change_percentage_24h_in_currency' in dictionary.keys() else APIHelper.SKIP
        total_supply = dictionary.get("total_supply") if dictionary.get("total_supply") else APIHelper.SKIP
        max_supply = dictionary.get("max_supply") if dictionary.get("max_supply") else APIHelper.SKIP
        circulating_supply = dictionary.get("circulating_supply") if dictionary.get("circulating_supply") else APIHelper.SKIP
        last_updated = APIHelper.RFC3339DateTime.from_value(dictionary.get("last_updated")).datetime if dictionary.get("last_updated") else APIHelper.SKIP
        # Return an object of this model
        return cls(current_price,
                   total_value_locked,
                   mcap_to_tvl_ratio,
                   fdv_to_tvl_ratio,
                   roi,
                   ath,
                   ath_change_percentage,
                   ath_date,
                   atl,
                   atl_change_percentage,
                   atl_date,
                   market_cap,
                   market_cap_rank,
                   fully_diluted_valuation,
                   market_cap_fdv_ratio,
                   total_volume,
                   high_24_h,
                   low_24_h,
                   price_change_24_h,
                   price_change_percentage_24_h,
                   price_change_percentage_7_d,
                   price_change_percentage_14_d,
                   price_change_percentage_30_d,
                   price_change_percentage_60_d,
                   price_change_percentage_200_d,
                   price_change_percentage_1_y,
                   market_cap_change_24_h,
                   market_cap_change_percentage_24_h,
                   price_change_percentage_1_h_in_currency,
                   price_change_percentage_24_h_in_currency,
                   price_change_percentage_7_d_in_currency,
                   price_change_percentage_14_d_in_currency,
                   price_change_percentage_30_d_in_currency,
                   price_change_percentage_60_d_in_currency,
                   price_change_percentage_200_d_in_currency,
                   price_change_percentage_1_y_in_currency,
                   market_cap_change_24_h_in_currency,
                   market_cap_change_percentage_24_h_in_currency,
                   total_supply,
                   max_supply,
                   circulating_supply,
                   last_updated)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'current_price={(self.current_price if hasattr(self, "current_price") else None)!r}, '
                f'total_value_locked={(self.total_value_locked if hasattr(self, "total_value_locked") else None)!r}, '
                f'mcap_to_tvl_ratio={(self.mcap_to_tvl_ratio if hasattr(self, "mcap_to_tvl_ratio") else None)!r}, '
                f'fdv_to_tvl_ratio={(self.fdv_to_tvl_ratio if hasattr(self, "fdv_to_tvl_ratio") else None)!r}, '
                f'roi={(self.roi if hasattr(self, "roi") else None)!r}, '
                f'ath={(self.ath if hasattr(self, "ath") else None)!r}, '
                f'ath_change_percentage={(self.ath_change_percentage if hasattr(self, "ath_change_percentage") else None)!r}, '
                f'ath_date={(self.ath_date if hasattr(self, "ath_date") else None)!r}, '
                f'atl={(self.atl if hasattr(self, "atl") else None)!r}, '
                f'atl_change_percentage={(self.atl_change_percentage if hasattr(self, "atl_change_percentage") else None)!r}, '
                f'atl_date={(self.atl_date if hasattr(self, "atl_date") else None)!r}, '
                f'market_cap={(self.market_cap if hasattr(self, "market_cap") else None)!r}, '
                f'market_cap_rank={(self.market_cap_rank if hasattr(self, "market_cap_rank") else None)!r}, '
                f'fully_diluted_valuation={(self.fully_diluted_valuation if hasattr(self, "fully_diluted_valuation") else None)!r}, '
                f'market_cap_fdv_ratio={(self.market_cap_fdv_ratio if hasattr(self, "market_cap_fdv_ratio") else None)!r}, '
                f'total_volume={(self.total_volume if hasattr(self, "total_volume") else None)!r}, '
                f'high_24_h={(self.high_24_h if hasattr(self, "high_24_h") else None)!r}, '
                f'low_24_h={(self.low_24_h if hasattr(self, "low_24_h") else None)!r}, '
                f'price_change_24_h={(self.price_change_24_h if hasattr(self, "price_change_24_h") else None)!r}, '
                f'price_change_percentage_24_h={(self.price_change_percentage_24_h if hasattr(self, "price_change_percentage_24_h") else None)!r}, '
                f'price_change_percentage_7_d={(self.price_change_percentage_7_d if hasattr(self, "price_change_percentage_7_d") else None)!r}, '
                f'price_change_percentage_14_d={(self.price_change_percentage_14_d if hasattr(self, "price_change_percentage_14_d") else None)!r}, '
                f'price_change_percentage_30_d={(self.price_change_percentage_30_d if hasattr(self, "price_change_percentage_30_d") else None)!r}, '
                f'price_change_percentage_60_d={(self.price_change_percentage_60_d if hasattr(self, "price_change_percentage_60_d") else None)!r}, '
                f'price_change_percentage_200_d={(self.price_change_percentage_200_d if hasattr(self, "price_change_percentage_200_d") else None)!r}, '
                f'price_change_percentage_1_y={(self.price_change_percentage_1_y if hasattr(self, "price_change_percentage_1_y") else None)!r}, '
                f'market_cap_change_24_h={(self.market_cap_change_24_h if hasattr(self, "market_cap_change_24_h") else None)!r}, '
                f'market_cap_change_percentage_24_h={(self.market_cap_change_percentage_24_h if hasattr(self, "market_cap_change_percentage_24_h") else None)!r}, '
                f'price_change_percentage_1_h_in_currency={(self.price_change_percentage_1_h_in_currency if hasattr(self, "price_change_percentage_1_h_in_currency") else None)!r}, '
                f'price_change_percentage_24_h_in_currency={(self.price_change_percentage_24_h_in_currency if hasattr(self, "price_change_percentage_24_h_in_currency") else None)!r}, '
                f'price_change_percentage_7_d_in_currency={(self.price_change_percentage_7_d_in_currency if hasattr(self, "price_change_percentage_7_d_in_currency") else None)!r}, '
                f'price_change_percentage_14_d_in_currency={(self.price_change_percentage_14_d_in_currency if hasattr(self, "price_change_percentage_14_d_in_currency") else None)!r}, '
                f'price_change_percentage_30_d_in_currency={(self.price_change_percentage_30_d_in_currency if hasattr(self, "price_change_percentage_30_d_in_currency") else None)!r}, '
                f'price_change_percentage_60_d_in_currency={(self.price_change_percentage_60_d_in_currency if hasattr(self, "price_change_percentage_60_d_in_currency") else None)!r}, '
                f'price_change_percentage_200_d_in_currency={(self.price_change_percentage_200_d_in_currency if hasattr(self, "price_change_percentage_200_d_in_currency") else None)!r}, '
                f'price_change_percentage_1_y_in_currency={(self.price_change_percentage_1_y_in_currency if hasattr(self, "price_change_percentage_1_y_in_currency") else None)!r}, '
                f'market_cap_change_24_h_in_currency={(self.market_cap_change_24_h_in_currency if hasattr(self, "market_cap_change_24_h_in_currency") else None)!r}, '
                f'market_cap_change_percentage_24_h_in_currency={(self.market_cap_change_percentage_24_h_in_currency if hasattr(self, "market_cap_change_percentage_24_h_in_currency") else None)!r}, '
                f'total_supply={(self.total_supply if hasattr(self, "total_supply") else None)!r}, '
                f'max_supply={(self.max_supply if hasattr(self, "max_supply") else None)!r}, '
                f'circulating_supply={(self.circulating_supply if hasattr(self, "circulating_supply") else None)!r}, '
                f'last_updated={(self.last_updated if hasattr(self, "last_updated") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'current_price={(self.current_price if hasattr(self, "current_price") else None)!s}, '
                f'total_value_locked={(self.total_value_locked if hasattr(self, "total_value_locked") else None)!s}, '
                f'mcap_to_tvl_ratio={(self.mcap_to_tvl_ratio if hasattr(self, "mcap_to_tvl_ratio") else None)!s}, '
                f'fdv_to_tvl_ratio={(self.fdv_to_tvl_ratio if hasattr(self, "fdv_to_tvl_ratio") else None)!s}, '
                f'roi={(self.roi if hasattr(self, "roi") else None)!s}, '
                f'ath={(self.ath if hasattr(self, "ath") else None)!s}, '
                f'ath_change_percentage={(self.ath_change_percentage if hasattr(self, "ath_change_percentage") else None)!s}, '
                f'ath_date={(self.ath_date if hasattr(self, "ath_date") else None)!s}, '
                f'atl={(self.atl if hasattr(self, "atl") else None)!s}, '
                f'atl_change_percentage={(self.atl_change_percentage if hasattr(self, "atl_change_percentage") else None)!s}, '
                f'atl_date={(self.atl_date if hasattr(self, "atl_date") else None)!s}, '
                f'market_cap={(self.market_cap if hasattr(self, "market_cap") else None)!s}, '
                f'market_cap_rank={(self.market_cap_rank if hasattr(self, "market_cap_rank") else None)!s}, '
                f'fully_diluted_valuation={(self.fully_diluted_valuation if hasattr(self, "fully_diluted_valuation") else None)!s}, '
                f'market_cap_fdv_ratio={(self.market_cap_fdv_ratio if hasattr(self, "market_cap_fdv_ratio") else None)!s}, '
                f'total_volume={(self.total_volume if hasattr(self, "total_volume") else None)!s}, '
                f'high_24_h={(self.high_24_h if hasattr(self, "high_24_h") else None)!s}, '
                f'low_24_h={(self.low_24_h if hasattr(self, "low_24_h") else None)!s}, '
                f'price_change_24_h={(self.price_change_24_h if hasattr(self, "price_change_24_h") else None)!s}, '
                f'price_change_percentage_24_h={(self.price_change_percentage_24_h if hasattr(self, "price_change_percentage_24_h") else None)!s}, '
                f'price_change_percentage_7_d={(self.price_change_percentage_7_d if hasattr(self, "price_change_percentage_7_d") else None)!s}, '
                f'price_change_percentage_14_d={(self.price_change_percentage_14_d if hasattr(self, "price_change_percentage_14_d") else None)!s}, '
                f'price_change_percentage_30_d={(self.price_change_percentage_30_d if hasattr(self, "price_change_percentage_30_d") else None)!s}, '
                f'price_change_percentage_60_d={(self.price_change_percentage_60_d if hasattr(self, "price_change_percentage_60_d") else None)!s}, '
                f'price_change_percentage_200_d={(self.price_change_percentage_200_d if hasattr(self, "price_change_percentage_200_d") else None)!s}, '
                f'price_change_percentage_1_y={(self.price_change_percentage_1_y if hasattr(self, "price_change_percentage_1_y") else None)!s}, '
                f'market_cap_change_24_h={(self.market_cap_change_24_h if hasattr(self, "market_cap_change_24_h") else None)!s}, '
                f'market_cap_change_percentage_24_h={(self.market_cap_change_percentage_24_h if hasattr(self, "market_cap_change_percentage_24_h") else None)!s}, '
                f'price_change_percentage_1_h_in_currency={(self.price_change_percentage_1_h_in_currency if hasattr(self, "price_change_percentage_1_h_in_currency") else None)!s}, '
                f'price_change_percentage_24_h_in_currency={(self.price_change_percentage_24_h_in_currency if hasattr(self, "price_change_percentage_24_h_in_currency") else None)!s}, '
                f'price_change_percentage_7_d_in_currency={(self.price_change_percentage_7_d_in_currency if hasattr(self, "price_change_percentage_7_d_in_currency") else None)!s}, '
                f'price_change_percentage_14_d_in_currency={(self.price_change_percentage_14_d_in_currency if hasattr(self, "price_change_percentage_14_d_in_currency") else None)!s}, '
                f'price_change_percentage_30_d_in_currency={(self.price_change_percentage_30_d_in_currency if hasattr(self, "price_change_percentage_30_d_in_currency") else None)!s}, '
                f'price_change_percentage_60_d_in_currency={(self.price_change_percentage_60_d_in_currency if hasattr(self, "price_change_percentage_60_d_in_currency") else None)!s}, '
                f'price_change_percentage_200_d_in_currency={(self.price_change_percentage_200_d_in_currency if hasattr(self, "price_change_percentage_200_d_in_currency") else None)!s}, '
                f'price_change_percentage_1_y_in_currency={(self.price_change_percentage_1_y_in_currency if hasattr(self, "price_change_percentage_1_y_in_currency") else None)!s}, '
                f'market_cap_change_24_h_in_currency={(self.market_cap_change_24_h_in_currency if hasattr(self, "market_cap_change_24_h_in_currency") else None)!s}, '
                f'market_cap_change_percentage_24_h_in_currency={(self.market_cap_change_percentage_24_h_in_currency if hasattr(self, "market_cap_change_percentage_24_h_in_currency") else None)!s}, '
                f'total_supply={(self.total_supply if hasattr(self, "total_supply") else None)!s}, '
                f'max_supply={(self.max_supply if hasattr(self, "max_supply") else None)!s}, '
                f'circulating_supply={(self.circulating_supply if hasattr(self, "circulating_supply") else None)!s}, '
                f'last_updated={(self.last_updated if hasattr(self, "last_updated") else None)!s})')



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from coingeckopublicapiv3.api_helper import APIHelper
from coingeckopublicapiv3.configuration import Server
from coingeckopublicapiv3.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from coingeckopublicapiv3.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from apimatic_core.authentication.multiple.or_auth_group import Or
from coingeckopublicapiv3.models.ping import Ping


class PingController(BaseController):

    """A Controller to access Endpoints in the coingeckopublicapiv3 API."""
    def __init__(self, config):
        super(PingController, self).__init__(config)

    def ping_server(self):
        """Does a GET request to /ping.

        This endpoint allows you to **check the API server status**

        Returns:
            Ping: Response from the API. Status OK

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/ping')
            .http_method(HttpMethodEnum.GET)
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Or(Single('apiKeyAuth'), Single('apiKeyQueryParam')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Ping.from_dictionary)
        ).execute()



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from coingeckopublicapiv3.api_helper import APIHelper
from coingeckopublicapiv3.models.company import Company


class CompaniesTreasury(object):

    """Implementation of the 'CompaniesTreasury' model.

    Attributes:
        total_holdings (float): total btc/eth holdings of companies
        total_value_usd (float): total btc/eth holdings value in usd
        market_cap_dominance (float): market cap dominance
        companies (List[Company]): The model property of type List[Company].

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "total_holdings": 'total_holdings',
        "total_value_usd": 'total_value_usd',
        "market_cap_dominance": 'market_cap_dominance',
        "companies": 'companies'
    }

    _optionals = [
        'total_holdings',
        'total_value_usd',
        'market_cap_dominance',
        'companies',
    ]

    def __init__(self,
                 total_holdings=APIHelper.SKIP,
                 total_value_usd=APIHelper.SKIP,
                 market_cap_dominance=APIHelper.SKIP,
                 companies=APIHelper.SKIP):
        """Constructor for the CompaniesTreasury class"""

        # Initialize members of the class
        if total_holdings is not APIHelper.SKIP:
            self.total_holdings = total_holdings 
        if total_value_usd is not APIHelper.SKIP:
            self.total_value_usd = total_value_usd 
        if market_cap_dominance is not APIHelper.SKIP:
            self.market_cap_dominance = market_cap_dominance 
        if companies is not APIHelper.SKIP:
            self.companies = companies 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        total_holdings = dictionary.get("total_holdings") if dictionary.get("total_holdings") else APIHelper.SKIP
        total_value_usd = dictionary.get("total_value_usd") if dictionary.get("total_value_usd") else APIHelper.SKIP
        market_cap_dominance = dictionary.get("market_cap_dominance") if dictionary.get("market_cap_dominance") else APIHelper.SKIP
        companies = None
        if dictionary.get('companies') is not None:
            companies = [Company.from_dictionary(x) for x in dictionary.get('companies')]
        else:
            companies = APIHelper.SKIP
        # Return an object of this model
        return cls(total_holdings,
                   total_value_usd,
                   market_cap_dominance,
                   companies)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'total_holdings={(self.total_holdings if hasattr(self, "total_holdings") else None)!r}, '
                f'total_value_usd={(self.total_value_usd if hasattr(self, "total_value_usd") else None)!r}, '
                f'market_cap_dominance={(self.market_cap_dominance if hasattr(self, "market_cap_dominance") else None)!r}, '
                f'companies={(self.companies if hasattr(self, "companies") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'total_holdings={(self.total_holdings if hasattr(self, "total_holdings") else None)!s}, '
                f'total_value_usd={(self.total_value_usd if hasattr(self, "total_value_usd") else None)!s}, '
                f'market_cap_dominance={(self.market_cap_dominance if hasattr(self, "market_cap_dominance") else None)!s}, '
                f'companies={(self.companies if hasattr(self, "companies") else None)!s})')



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class IntervalEnum(object):

    """Implementation of the 'interval' enum.

    Attributes:
        DAILY: The enum member of type str.

    """
    DAILY = 'daily'




# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from coingeckopublicapiv3.api_helper import APIHelper
from coingeckopublicapiv3.configuration import Server
from coingeckopublicapiv3.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from coingeckopublicapiv3.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from apimatic_core.authentication.multiple.or_auth_group import Or
from coingeckopublicapiv3.models.coins_data_base import CoinsDataBase
from coingeckopublicapiv3.models.coins_market_chart import CoinsMarketChart
from coingeckopublicapiv3.models.coins_market_chart_range import CoinsMarketChartRange


class ContractController(BaseController):

    """A Controller to access Endpoints in the coingeckopublicapiv3 API."""
    def __init__(self, config):
        super(ContractController, self).__init__(config)

    def coins_contract_address(self,
                               id,
                               contract_address):
        """Does a GET request to /coins/{id}/contract/{contract_address}.

        This endpoint allows you to **query all the metadata (image, websites,
        socials, description, contract address, etc.) and market data (price,
        ATH, exchange tickers, etc.) of a coin from the CoinGecko coin page
        based on an asset platform and a particular token contract address**

        Args:
            id (str): asset platform ID <br> *refers to
                [`/asset_platforms`](/reference/asset-platforms-list).
            contract_address (str): the contract address of token

        Returns:
            CoinsDataBase: Response from the API. Get current data for a coin

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/coins/{id}/contract/{contract_address}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .template_param(Parameter()
                            .key('contract_address')
                            .value(contract_address)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Or(Single('apiKeyAuth'), Single('apiKeyQueryParam')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CoinsDataBase.from_dictionary)
        ).execute()

    def contract_address_market_chart(self,
                                      id,
                                      contract_address,
                                      vs_currency,
                                      days,
                                      interval=None,
                                      precision=None):
        """Does a GET request to /coins/{id}/contract/{contract_address}/market_chart.

        This endpoint allows you to **get the historical chart data including
        time in UNIX, price, market cap and 24hr volume based on asset
        platform and particular token contract address**

        Args:
            id (str): asset platform ID <br> *refers to
                [`/asset_platforms`](/reference/asset-platforms-list).
            contract_address (str): the contract address of token
            vs_currency (str): target currency of market data <br> *refers to
                [`/simple/supported_vs_currencies`](/reference/simple-supported
                -currencies).
            days (str): data up to number of days ago <br> You may use any
                integer for number of days
            interval (IntervalEnum, optional): data interval, leave empty for
                auto granularity Possible value: daily
            precision (PrecisionEnum, optional): decimal place for currency
                price value

        Returns:
            CoinsMarketChart: Response from the API. Get historical market
                data include price, market cap, and 24hr volume

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/coins/{id}/contract/{contract_address}/market_chart')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .template_param(Parameter()
                            .key('contract_address')
                            .value(contract_address)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('vs_currency')
                         .value(vs_currency))
            .query_param(Parameter()
                         .key('days')
                         .value(days))
            .query_param(Parameter()
                         .key('interval')
                         .value(interval))
            .query_param(Parameter()
                         .key('precision')
                         .value(precision))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Or(Single('apiKeyAuth'), Single('apiKeyQueryParam')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CoinsMarketChart.from_dictionary)
        ).execute()

    def contract_address_market_chart_range(self,
                                            id,
                                            contract_address,
                                            vs_currency,
                                            mfrom,
                                            to,
                                            precision=None):
        """Does a GET request to /coins/{id}/contract/{contract_address}/market_chart/range.

        This endpoint allows you to **get the historical chart data within
        certain time range in UNIX along with price, market cap and 24hr
        volume based on asset platform and particular token contract address**

        Args:
            id (str): asset platform ID <br> *refers to
                [`/asset_platforms`](/reference/asset-platforms-list)
            contract_address (str): the contract address of token
            vs_currency (str): target currency of market data <br> *refers to
                [`/simple/supported_vs_currencies`](/reference/simple-supported
                -currencies).
            mfrom (int): starting date in UNIX timestamp
            to (int): ending date in UNIX timestamp
            precision (PrecisionEnum, optional): decimal place for currency
                price value

        Returns:
            CoinsMarketChartRange: Response from the API. Get historical
                market data include price, market cap, and 24hr volume

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/coins/{id}/contract/{contract_address}/market_chart/range')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .template_param(Parameter()
                            .key('contract_address')
                            .value(contract_address)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('vs_currency')
                         .value(vs_currency))
            .query_param(Parameter()
                         .key('from')
                         .value(mfrom))
            .query_param(Parameter()
                         .key('to')
                         .value(to))
            .query_param(Parameter()
                         .key('precision')
                         .value(precision))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Or(Single('apiKeyAuth'), Single('apiKeyQueryParam')))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CoinsMarketChartRange.from_dictionary)
        ).execute()



# -*- coding: utf-8 -*-

"""
coingeckopublicapiv3

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from coingeckopublicapiv3.api_helper import APIHelper
from coingeckopublicapiv3.models.market_cap_percentage import MarketCapPercentage
from coingeckopublicapiv3.models.total_market_cap import TotalMarketCap
from coingeckopublicapiv3.models.total_volume_2 import TotalVolume2


class Data3(object):

    """Implementation of the 'Data3' model.

    Attributes:
        active_cryptocurrencies (float): number of active cryptocurrencies
        upcoming_icos (float): number of upcoming icos
        ongoing_icos (float): number of ongoing icos
        ended_icos (float): number of ended icos
        markets (float): number of exchanges
        total_market_cap (TotalMarketCap): cryptocurrencies total market cap
        total_volume (TotalVolume2): cryptocurrencies total volume
        market_cap_percentage (MarketCapPercentage): cryptocurrencies market
            cap percentage
        market_cap_change_percentage_24_h_usd (float): cryptocurrencies market
            cap change percentage in 24 hours in usd
        updated_at (float): The model property of type float.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "active_cryptocurrencies": 'active_cryptocurrencies',
        "upcoming_icos": 'upcoming_icos',
        "ongoing_icos": 'ongoing_icos',
        "ended_icos": 'ended_icos',
        "markets": 'markets',
        "total_market_cap": 'total_market_cap',
        "total_volume": 'total_volume',
        "market_cap_percentage": 'market_cap_percentage',
        "market_cap_change_percentage_24_h_usd": 'market_cap_change_percentage_24h_usd',
        "updated_at": 'updated_at'
    }

    _optionals = [
        'active_cryptocurrencies',
        'upcoming_icos',
        'ongoing_icos',
        'ended_icos',
        'markets',
        'total_market_cap',
        'total_volume',
        'market_cap_percentage',
        'market_cap_change_percentage_24_h_usd',
        'updated_at',
    ]

    def __init__(self,
                 active_cryptocurrencies=APIHelper.SKIP,
                 upcoming_icos=APIHelper.SKIP,
                 ongoing_icos=APIHelper.SKIP,
                 ended_icos=APIHelper.SKIP,
                 markets=APIHelper.SKIP,
                 total_market_cap=APIHelper.SKIP,
                 total_volume=APIHelper.SKIP,
                 market_cap_percentage=APIHelper.SKIP,
                 market_cap_change_percentage_24_h_usd=APIHelper.SKIP,
                 updated_at=APIHelper.SKIP):
        """Constructor for the Data3 class"""

        # Initialize members of the class
        if active_cryptocurrencies is not APIHelper.SKIP:
            self.active_cryptocurrencies = active_cryptocurrencies 
        if upcoming_icos is not APIHelper.SKIP:
            self.upcoming_icos = upcoming_icos 
        if ongoing_icos is not APIHelper.SKIP:
            self.ongoing_icos = ongoing_icos 
        if ended_icos is not APIHelper.SKIP:
            self.ended_icos = ended_icos 
        if markets is not APIHelper.SKIP:
            self.markets = markets 
        if total_market_cap is not APIHelper.SKIP:
            self.total_market_cap = total_market_cap 
        if total_volume is not APIHelper.SKIP:
            self.total_volume = total_volume 
        if market_cap_percentage is not APIHelper.SKIP:
            self.market_cap_percentage = market_cap_percentage 
        if market_cap_change_percentage_24_h_usd is not APIHelper.SKIP:
            self.market_cap_change_percentage_24_h_usd = market_cap_change_percentage_24_h_usd 
        if updated_at is not APIHelper.SKIP:
            self.updated_at = updated_at 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        active_cryptocurrencies = dictionary.get("active_cryptocurrencies") if dictionary.get("active_cryptocurrencies") else APIHelper.SKIP
        upcoming_icos = dictionary.get("upcoming_icos") if dictionary.get("upcoming_icos") else APIHelper.SKIP
        ongoing_icos = dictionary.get("ongoing_icos") if dictionary.get("ongoing_icos") else APIHelper.SKIP
        ended_icos = dictionary.get("ended_icos") if dictionary.get("ended_icos") else APIHelper.SKIP
        markets = dictionary.get("markets") if dictionary.get("markets") else APIHelper.SKIP
        total_market_cap = TotalMarketCap.from_dictionary(dictionary.get('total_market_cap')) if 'total_market_cap' in dictionary.keys() else APIHelper.SKIP
        total_volume = TotalVolume2.from_dictionary(dictionary.get('total_volume')) if 'total_volume' in dictionary.keys() else APIHelper.SKIP
        market_cap_percentage = MarketCapPercentage.from_dictionary(dictionary.get('market_cap_percentage')) if 'market_cap_percentage' in dictionary.keys() else APIHelper.SKIP
        market_cap_change_percentage_24_h_usd = dictionary.get("market_cap_change_percentage_24h_usd") if dictionary.get("market_cap_change_percentage_24h_usd") else APIHelper.SKIP
        updated_at = dictionary.get("updated_at") if dictionary.get("updated_at") else APIHelper.SKIP
        # Return an object of this model
        return cls(active_cryptocurrencies,
                   upcoming_icos,
                   ongoing_icos,
                   ended_icos,
                   markets,
                   total_market_cap,
                   total_volume,
                   market_cap_percentage,
                   market_cap_change_percentage_24_h_usd,
                   updated_at)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'active_cryptocurrencies={(self.active_cryptocurrencies if hasattr(self, "active_cryptocurrencies") else None)!r}, '
                f'upcoming_icos={(self.upcoming_icos if hasattr(self, "upcoming_icos") else None)!r}, '
                f'ongoing_icos={(self.ongoing_icos if hasattr(self, "ongoing_icos") else None)!r}, '
                f'ended_icos={(self.ended_icos if hasattr(self, "ended_icos") else None)!r}, '
                f'markets={(self.markets if hasattr(self, "markets") else None)!r}, '
                f'total_market_cap={(self.total_market_cap if hasattr(self, "total_market_cap") else None)!r}, '
                f'total_volume={(self.total_volume if hasattr(self, "total_volume") else None)!r}, '
                f'market_cap_percentage={(self.market_cap_percentage if hasattr(self, "market_cap_percentage") else None)!r}, '
                f'market_cap_change_percentage_24_h_usd={(self.market_cap_change_percentage_24_h_usd if hasattr(self, "market_cap_change_percentage_24_h_usd") else None)!r}, '
                f'updated_at={(self.updated_at if hasattr(self, "updated_at") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'active_cryptocurrencies={(self.active_cryptocurrencies if hasattr(self, "active_cryptocurrencies") else None)!s}, '
                f'upcoming_icos={(self.upcoming_icos if hasattr(self, "upcoming_icos") else None)!s}, '
                f'ongoing_icos={(self.ongoing_icos if hasattr(self, "ongoing_icos") else None)!s}, '
                f'ended_icos={(self.ended_icos if hasattr(self, "ended_icos") else None)!s}, '
                f'markets={(self.markets if hasattr(self, "markets") else None)!s}, '
                f'total_market_cap={(self.total_market_cap if hasattr(self, "total_market_cap") else None)!s}, '
                f'total_volume={(self.total_volume if hasattr(self, "total_volume") else None)!s}, '
                f'market_cap_percentage={(self.market_cap_percentage if hasattr(self, "market_cap_percentage") else None)!s}, '
                f'market_cap_change_percentage_24_h_usd={(self.market_cap_change_percentage_24_h_usd if hasattr(self, "market_cap_change_percentage_24_h_usd") else None)!s}, '
                f'updated_at={(self.updated_at if hasattr(self, "updated_at") else None)!s})')
